# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module baseclasses
##   Classes de base utilises par pycc
## ----------------------------------------------------------------------

class PyccError(Exception):
  """ Exceptions de pycc.
  """
  def __init__(self, msg=''):
    self.message = msg
  
  def __str__(self):
    return self.message

import types
_unicode = types.UnicodeType  
import time

_version = 0.8 # Pour rtrocompatibilit

class Norloge(object):
  """ Objet pour la manipulation d'horloges-rfrences gnriques. """
  
  def __init__(self, timestamp=None, trib="", num=0, tuple=None):
    """ Construit l'horloge de deux manires possibles :
      -  partir de son timestamp (format YYYYMMDDhhmmss ou YYYY-MM-DD hh:mm:ss),
      de la tribune de rfrence, et d'un ventuel indice numrique (pour les posts multiples
       la mme seconde).
      -  partir d'un tuple (h,m,s,indice,jour,mois,annee,tribune) tel que rcupr du
      parsing d'un post par exemple.
    """
    if tuple is not None:
      timestamp = tuple[6]+tuple[5]+tuple[4]+tuple[0]+tuple[1]+tuple[2]
      num = tuple[3]
      trib = tuple[7]
    if len(timestamp) > 14:
      self._timestamp = timestamp[:4]+timestamp[5:7]+timestamp[8:10]+timestamp[11:13]+timestamp[14:16]+timestamp[17:19]
    else:
      self._timestamp = timestamp
    if type(self._timestamp) != _unicode:
      self._timestamp = unicode(self._timestamp, 'utf-8')
    self.trib = trib
    self.num = num
  
  def __cmp__(self, other):
    if self._timestamp > other._timestamp:
      return 1
    if self._timestamp == other._timestamp:
      if other.num == 0 or other.num == self.num or (other.num == 1 and self.num == 0):
        return 0
      if self.num < other.num:
        return 1
    return -1
  
  def __str__(self):
    return self.getFullRef()
  
  def same_time(self, atime):
    return self._timestamp == atime._timestamp
  
  def cmp_ref(self, ref):
    """ Compare temporellement l'horloge avec la rfrence passe en paramtre
        (au format interne pycc)
    """
    # print "### CMP:", self._timestamp, ref
    pos = ref.find("@")
    if pos < 0:
      trib = self.trib
      pos = len(ref)
    else:
      trib = ref[pos+1:]
    if _version > 0.8:
      timestamp = ref[3:17]
      num = int(ref[17:pos])
    else:
      # Compatibilit avec l'ancien format de ref (pycc 0.8.x)
      timestamp = self._timestamp[:8]+ref[3:7]
      if ref[7:9] == "--":
        timestamp += self._timestamp[12:14]
      else:
        timestamp += ref[7:9]
      num = int(ref[9:pos])
    # print "### CMP:", self._timestamp, timestamp
    return self.__cmp__(Norloge(timestamp, trib, num))
  
  def getTuple(self):
    return (self.hour(), self.min(), self.sec(), self.num, self.day(), self.mon(), self.year(), self.trib)
  
  def year(self):
    return self._timestamp[:4]
  
  def mon(self):
    return self._timestamp[4:6]
  
  def day(self):
    return self._timestamp[6:8]
  
  def hour(self):
    return self._timestamp[8:10]
  
  def min(self):
    return self._timestamp[10:12]
  
  def sec(self):
    return self._timestamp[12:14]
  
  def inc(self):
    """ Retourne l'indice de l'horloge, en cas d'horloges multiples  la mme seconde """
    if self.num == 0:
      return ""
    if self.num <= 3:
      return ur""[self.num-1]
    return u"^%d" % self.num
  
  def getDate(self):
    """ Renvoie la date de l'horloge au format jj/mm """
    return u"%s/%s" % (self.day(), self.mon())
  
  def getFullDate(self):
    """ Renvoie la date de l'horloge au format jj/mm/aaaa """
    return u"%s/%s/%s" % (self.day(), self.mon(), self.year())
  
  def getTime(self):
    """ Renvoie l'heure de l'horloge au format hh:mm:ss """
    if self.sec() == "--":
      return u"%s:%s" % (self.hour(), self.min())
    else:
      return u"%s:%s:%s" % (self.hour(), self.min(), self.sec())
  
  def getRef(self):
    """ Renvoie la rfrence de base au format hh:mm:ssi """
    return u"%s:%s:%s%s" % (self.hour(), self.min(), self.sec(), self.inc())
  
  def getFullRef(self):
    """ Renvoie la rfrence complte, au format jj/mm#hh:mm:ssi@tribune """
    return u"%s#%s@%s" % (self.getDate(), self.getRef(), self.trib)
  
  def getCtxtRef(self, trib=None):
    """ Renvoie une rfrence contextuelle : la date n'est crite que si diffrente
        de la date actuelle, et la tribune n'est crite que si diffrente de celle
        passe en paramtre
    """
    curdate = u"%s" % time.strftime("%d/%m")
    if curdate != self.getDate():
      date = u"%s#" % self.getDate()
    else:
      date = u""
    if trib is None or trib == self.trib:
      tribref = u""
    else:
      tribref = u"@%s" % self.trib
    return date + self.getRef() + tribref
  
  def intRef(self):
    """ Renvoie la rfrence de l'horloge au format interne pycc """
    return u"ref%s%s@%s" % (self._timestamp, self.num, self.trib)
  
  def allIntRefs(self):
    """ Renvoie toutes les rfrences au format interne pycc susceptibles de pointer
        sur la mme horloge.
    """
    if self.num == 0:
      other = u"1"
    else:
      other = u"0"
    ref = self.intRef()
    pos = ref.find("@")
    if _version > 0.8:
      return [ref, ref[:17]+other+ref[pos:], ref[:15]+u"--0"+ref[pos:]]
    else: # rtro-compatibilit
      return [ref[11:pos], ref[11:17]+other, ref[11:15]+u"--0"]
  

class Post(object):
  """ Objet reprsentant un post sur une tribune. """
  
  def __init__(self, id='', time='', login='', info='', message='', msg_tags={}, trib=''):
    self.id = int(id)
    self.time = Norloge(time, trib)
    self.info = info
    self.login = login
    self.message = message
    self.tags = msg_tags
    self.original_text = ''
    self.status = []
    self.next = None
    self.prev = None
    if _version <= 0.8:
      self.num = self.time.num # gard pour rtro-compatibilit
  
  def clone(self):
    """ Renvoie une copie de l'objet """
    import copy
    npost = copy.copy(self)
    npost.tags = self.tags.copy()
    npost.time = Norloge(tuple=self.time.getTuple())
    return npost
  
  def __str__(self):
    return repr(u"Post %s:%s (%s - %s)" % (self.getTrib(), self.id, self.time.getRef(), self.time.getFullDate()))
  
  def __cmp__(self, other):
    return self.id - other.id
  
  def disp(self, name):
    """ Mthode pour la prsentation  l'affichage des champs du post """
    if name == 'id':
      return str(self.id)
    elif name == 'trib':
      return self.time.trib
    elif name == 'time':
      return self.time.getTime()
    elif name == 'date':
      return self.time.getFullDate()
    elif name == 'info':
      if self.info == "":
        return ur"[vide]"
      else:
        return self.info[:10]
    elif name == 'ua':
      return self.info
    elif name == 'login':
      if self.login == u'Anonyme':
        return u""
      else:
        return self.login
    else:
      if hasattr(self, name):
        return eval("self." + name)
      else:
        return u""
  
  def set(self, name, value):
    if name == 'id':
      self.id = value
    elif name == 'time':
      self.time = value
    elif name == 'info':
      self.info = value
    elif name == 'login':
      self.login = value
    elif name == 'message':
      self.message = value
    elif name == 'num':
      self.time.num = value
      if _version <= 0.8:
        self.num = value # gard pour rtro-compatibilit
  
  def compare_time(self, ref):
    """ Compare l'horloge du post avec la rfrence passe en paramtre (au format interne pycc) """
    return self.time.cmp_ref(ref)
  
  def same_time(self, apost):
    return self.time.same_time(apost.time)
  
  def getTrib(self):
    """ Renvoie la tribune  laquelle le post est rattach """
    return self.time.trib
  
  def getAllRefs(self):
    """ Renvoie la liste des rfrences (format interne) susceptibles de pointer sur ce post """
    return self.time.allIntRefs()
  
  def dispTimeRef(self, trib=None):
    """ Renvoie l'horloge-rfrence au post """
    return self.time.getCtxtRef()
  
  def dispDateTimeRef(self):
    """ Renvoie l'horloge-rfrence complte, avec la date """
    return self.time.getFullRef()
  