# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module threads
##   Dfinitions des types de threads utiliss par pycc
## ----------------------------------------------------------------------

# import sys
# sys.path.append('..')

import logging
log = logging.getLogger('pycc')

from threading import *
from Queue import Queue


class QueueThread(Thread):
  """ Thread de base pour grer une queue FIFO.
  Lorsque le thread est lanc (par la mthode 'run'), il attend de recevoir des
  donnes dans sa queue et les traite immdiatement en faisant appel  sa
  mthode 'process' (cette dernire doit donc tre dfinie dans les classes drives).
  """
  def __init__(self, group=None, target=None, name=None, args=(), kwargs={}):
    Thread.__init__(self, group, target, name, args, kwargs)
    self.queue = Queue(0)
    self.running = 0
  
  def run(self):
    self.running = 1
    while self.running:
      args = self.queue.get()
      if args[-1] == "__stop__":
        return
      self.process(args)
  
  def process(self, args):
    pass # Mthode abstraite
  
  def interrupt(self):
    pass # Ne fait rien, gard pour compatibilit avec l'interface des GenericThreads
  
  def quit(self):
    self.running = 0
    self.queue.put(["__stop__"])


class PollThread(Thread):
  """ Thread de base pour faire de la scrutation.
  Lorsque le thread est lanc (par appel  la mthode 'run'), la mthode 'process' est appele toutes les 'refresh' secondes.
  La mthode 'interrupt' permet de forcer l'appel  'process' alors que le thread est en attente.
  """
  def __init__(self, refresh, group=None, target=None, name=None, args=(), kwargs={}):
    Thread.__init__(self, group, target, name, args, kwargs)
    self.refresh = refresh
    self.cartouche = Event()
    self.running = 0
  
  def run(self):
    self.running = 1
    while self.running:
      self.process()
      self.cartouche.clear()
      self.cartouche.wait(self.refresh)
    self.running = 0
  
  def process(self):
    pass # A implmenter dans les classes drives
  
  def interrupt(self):
    self.cartouche.set()
  
  def quit(self):
    self.running = 0
    self.cartouche.set()
  

class GenericThread(object):
  """ Classe gnrique pour manipuler facilement des PollThreads ou des QueueThreads.
  Implmente les mthodes suivantes :
    start, stop, restart, interrupt, isRunning, setDelay.
  La mthode 'process' contient le traitement, elle est abstraite et doit tre
  implmente dans les classes drives.
  """
  def __init__(self, name, refresh=20, type="poll"):
    self.name = name
    self.type = type  # soit "poll", soit "queue"
    self._initialize(refresh)
  
  def _initialize(self, refresh):
    if self.type == "poll":
      self._thread = PollThread(refresh)
    else:
      self._thread = QueueThread()
    self._thread.process = self.process
  
  def start(self):
    log.info("Lancement thread %s" % self.name)
    if self.isRunning():
      log.info("Thread %s dj lanc" % self.name)
      return
    self._thread.start()
  
  def restart(self):
    if self.type == "poll":
      delay = self._thread.refresh
    else:
      delay = 0
    self.stop()
    self._initialize(delay)
    self.start()
  
  def stop(self):
    log.info("Arrt du thread %s" % self.name)
    if not self.isRunning():
      log.info("Thread %s dj arrt" % self.name)
      return
    self._thread.quit()
  
  def isRunning(self):
    return self._thread.running
  
  def interrupt(self):
    self._thread.interrupt()
  
  def setDelay(self, delay):
    self._thread.refresh = delay
    self._thread.interrupt()
  
  def write(self, args):
    # Pour crire dans la queue, en cas de queuethread
    self._thread.queue.put(args)
  
  def process(self):
    log.warning("Thread %s: pas de mthode process dfinie" % self.name)
    pass # A implmenter dans les classes drives
  
