# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module cointklib.actions
##   Widgets d'dition de tout ce qui touche aux vnements (key, click)
## ----------------------------------------------------------------------

from Tkinter import *
import tkMessageBox
import Pmw
import logging
import sys, os, os.path

sys.path.append('..')

# from coinlib.utils import *
from coinlib.persistants import *
from basicwidgets import *
from widgets import *
from dialogs import CoinDialog, SelectDialog

log = logging.getLogger('pycc')


class EventDialog(CoinDialog):
  
  def __init__(self, parent, object, extra_arg):
    self.object = object
    self.extra_arg = extra_arg
    CoinDialog.__init__(self, parent, "Nouvelle action", [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    self.w_event = EventEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_event.pack()
  
  def apply(self):
    self.w_event.apply()
    self.parent.selected = self.w_event.get()

class EventEditor(CoinObjectEditor):
  
  def body(self):
    ctxt, pycc = self.extra_arg
    actlist = {}
    for act, descr in actions_descr.items():
      if act in actions_ctx[ctxt]:
        actlist[act] = descr
    self.w_actionlist = ComboField(self._fr, "Action", actlist, self.object.action.type)
    self.w_actionlist.pack(side=TOP, fill=X, anchor=NW)
    self.w_actionlist.bind_onchange(self.set_action)
    self.w_action = None
    if ctxt == PINNI or ctxt == MENU:
      self.w_addtopp = BoolField(self._fr, "Ajouter automatiquement au Post-Processor", self.object.addtopp)
      self.w_addtopp.pack(side=BOTTOM, fill=X, anchor=SW)
      self.w_runmode = ComboField(self._fr, "Appliquer", [ACTIONMODE_MULTI, ACTIONMODE_CTXT, ACTIONMODE_ONCE], self.object.runmode)
      self.w_runmode.pack(side=BOTTOM, fill=X, anchor=SW)
    else:
      self.w_addtopp = None
      self.w_runmode = None
    self.set_action(self.object.action.type, self.object.action)
  
  def set_action(self, type=None, act_obj=None):
    if self.w_action is not None:
      self.w_action.pack_forget()
    if type is None:
      type = self.w_actionlist.get()
    if act_obj is None:
      act_obj = buildNewAction(type)
    self.w_action = ActionEditor(self._fr, act_obj, self.extra_arg, border=2, relief=GROOVE)
    self.w_action.pack(fill=X, pady=5)
  
  def apply(self):
    self.w_action.apply()
    self.object.action = self.w_action.object
    if self.w_addtopp is not None:
      self.object.addtopp = self.w_addtopp.get()
    if self.w_runmode is not None:
      self.object.runmode = self.w_runmode.get()
  

class EventsEditor(CoinObjectEditor):
  
  def __init__(self, parent, title, object, ctxt, pycc, scrolled=0, **kwargs):
    self.title = title
    self.pycc = pycc
    self.context = ctxt
    CoinObjectEditor.__init__(self, parent, object, scrolled=0, **kwargs)
  
  def body(self):
    Label(self._fr, text=self.title, background=COLOR_TITLE).pack(fill=X, anchor=NW)
    self.fr = Frame(self._fr)
    ifr = Frame(self._fr)
    self.eventlist = SortedListBox(ifr, self.object.get().keys(), height=10, width=25)
    self.eventlist.pack(side=TOP, fill=BOTH, expand=1, anchor=NW)
    self.eventlist.bind_select(self.select)
    bbox = Frame(ifr)
    Button(bbox, text="Nouveau...", border=1, command=self.new).pack(side=LEFT)
    Button(bbox, text="Supprimer", border=1, command=self.delete).pack(side=LEFT)
    bbox.pack(anchor=SW, fill=X)
    ifr.pack(side=LEFT, anchor=NW, fill=Y, expand=1)
    self.w_events = {}
    for k in self.object.get().keys():
      self.w_events[k] = EventEditor(self.fr, self.object.get(k), extra_arg=(self.context, self.pycc))
    self.fr.pack(anchor=NW)
  
  def select(self):
    for k, w in self.w_events.items():
      w.pack_forget()
    cur_k = self.eventlist.get()
    if not cur_k: return
    self.w_events[cur_k].pack(side=LEFT, padx=5, pady=5, anchor=NW)
  
  def eventSelector(self):
    pass # abstract method, must open the event selector window
  
  def unbindEvents(self):
    pass # abstract method, must do unbinding of events set
  
  def bindEvents(self):
    pass # abstract method, must do binding of events set
  
  def select(self):
    for w in self.w_events.values():
      w.pack_forget()
    event = self.eventlist.get()
    if not event: return
    self.w_events[event].pack(side=LEFT, padx=5, pady=5)
  
  def new(self):
    self.newevent = None
    self.eventSelector()
    if self.newevent != None:
      if self.newevent in self.w_events.keys():
        tkMessageBox.showwarning(self.title, "'%s' est dj dfini" % self.newevent, parent=self)
        return
      newevent = PyccEvent(ActionPost())
      self.w_events[self.newevent] = EventEditor(self.fr, newevent, extra_arg=(self.context, self.pycc))
      self.eventlist.add(self.newevent)
      self.eventlist.select(self.newevent)
  
  def delete(self):
    event = self.eventlist.get()
    if event is None: return
    self.w_events[event].pack_forget()
    del self.w_events[event]
    self.eventlist.remove(event)
  
  def apply(self):
    self.unbindEvents()
    for event in self.w_events.keys():
      self.w_events[event].apply()
      self.object.set(event, self.w_events[event].get())
    tmp_events = []
    for event in self.object.get().keys():
      if event not in self.w_events.keys():
        tmp_events.append(event)
    for event in tmp_events:
      self.object.delete(event)
    self.bindEvents()

  
# -----------------------------------------------
# Slecteur de Raccourcis clavier
# -----------------------------------------------

class KeyEventsEditor(EventsEditor):
  
  def __init__(self, parent, keyset, ctxt, pycc):
    EventsEditor.__init__(self, parent, "Actions Clavier", keyset, ctxt, pycc)
  
  def unbindEvents(self):
    if self.context == PALMI:
      self.pycc.palmi.unbindKeys()
    else:
      for bouchot in self.pycc.bouchots.values():
        if bouchot.trib is not None:
          if bouchot.trib.loaded:
            bouchot.trib.unbindKeys()
  
  def bindEvents(self):
    if self.context == PALMI:
      self.pycc.palmi.bindKeys()
    else:
      for bouchot in self.pycc.bouchots.values():
        if bouchot.trib is not None:
          if bouchot.trib.loaded:
            bouchot.trib.bindKeys()
  
  def eventSelector(self):
    KeySelector(self, modal=1)
  
class KeySelector(CoinDialog):
  
  def body(self, master):
    Label(master, text="Entrez la combinaison de touches  ajouter", background="#fcfcda").pack(side=TOP)
    self.hotkey = Entry(master)
    self.hotkey.bind("<Key>", self.dispKey)
    self.hotkey.pack(side=TOP, pady=5)
    self.reset()
    self.hotkey.focus()
    self.hotkey.config(state=DISABLED)
  
  def apply(self):
    self.parent.newevent = self.getKey()
  
  def dispKey(self, event):
    self.hotkey.config(state=NORMAL)
    k = str(event.keysym)
    if k in erasekeys:
      self.reset()
      self.hotkey.config(state=DISABLED)
      return
    if k in ignorekeys: return
    if k[-2:] == "_R" or k[-2:] == "_L":
      k = k[:-2]
    if k in modifkeys:
      if k not in self.modifiers:
        self.modifiers.append(k)
    else:
      self.modifiers.sort()
      self.hotkey.delete(0, END)
      if len(k) == 1 and "Shift" in self.modifiers:
        k = k.upper()
        self.modifiers.remove("Shift")
      if len(self.modifiers) == 0:
        self.hotkey.config(state=DISABLED)
        return
      self.curkey = self.modifiers + [k]
      self.hotkey.insert(END, self.getKey())
      self.modifiers = []
    self.hotkey.config(state=DISABLED)
  
  def reset(self):
    self.hotkey.delete(0, END)
    self.curkey = []
    self.modifiers = []
  
  def getKey(self):
    return "-".join(self.curkey)
  
  
# -----------------------------------------------
# Slecteur de clicks souris
# -----------------------------------------------

class ClickEventsEditor(EventsEditor):

  def __init__(self, parent, clickset, pycc):
    EventsEditor.__init__(self, parent, "Actions Souris", clickset, PINNI, pycc)
  
  def unbindEvents(self):
    for bouchot in self.pycc.bouchots.values():
      if bouchot.trib is not None:
        if bouchot.trib.loaded:
          bouchot.trib.unbindClicks()
  
  def bindEvents(self):
    for bouchot in self.pycc.bouchots.values():
      if bouchot.trib is not None:
        if bouchot.trib.loaded:
          bouchot.trib.bindClicks()
  
  def eventSelector(self):
    ClickSelector(self, modal=1)
  
class ClickSelector(CoinDialog):

  def body(self, master):
    Label(master, text="Entrez le nouvel vnement souris  ajouter", background="#fcfcda").pack(side=TOP)
    Label(master, text="Modifieur :", justify=LEFT, anchor=W).pack(side=TOP, anchor=W)
    fr = Frame(master, relief=FLAT, border=0)
    self.shift = BoolField(fr, "Shift")
    self.shift.pack(side=LEFT)
    self.control = BoolField(fr, "Control")
    self.control.pack(side=LEFT)
    self.alt = BoolField(fr, "Alt")
    self.alt.pack(side=LEFT)
    fr.pack(side=TOP, anchor=W)
    self.button = ComboField(master, "Bouton :", {1:'Gauche', 2:'Milieu', 3:'Droit'}, 1)
    self.button.pack(side=TOP, anchor=W)
    self.double = BoolField(master, "Double-click")
    self.double.pack(side=TOP, anchor=W)
  
  def apply(self):
    self.parent.newevent = self.getClick()
  
  def validate(self):
    if len(self.getClick()) < 6:
      tkMessageBox.showwarning("DaHotClicks", "Vous devez obligatoirement slectionner au moins un modifieur", parent=self)
      return 0
    return 1
  
  def getClick(self):
    modifiers = []
    if self.double.get():
      modifiers.append("Double")
    if self.alt.get():
      modifiers.append("Alt")
    if self.control.get():
      modifiers.append("Control")
    if self.shift.get():
      modifiers.append("Shift")
    modifiers += ["ButtonRelease", "%s" % self.button.get()]
    return "-".join(modifiers)


# -----------------------------------------------
# Slecteur d'entres de menu
# -----------------------------------------------

class MenuEventsEditor(EventsEditor):
  
  def __init__(self, parent, menuset, pycc):
    EventsEditor.__init__(self, parent, "Menu contextuel", menuset, MENU, pycc)
  
  def eventSelector(self):
    MenuSelector(self, modal=1)

class MenuSelector(CoinDialog):

  def body(self, master):
    Label(master, text="Intitul de la nouvelle entre menu  ajouter :", background="#fcfcda").pack(side=TOP)
    self.w_menu = TextField(master, '')
    self.w_menu.pack(side=TOP)
  
  def apply(self):
    self.parent.newevent = self.w_menu.get()
  


## -----------------------------------------------
## Gestion des filtres / post-processor
## -----------------------------------------------

class FilterEditor(CoinObjectEditor):
  
  def body(self):
    Label(self._fr, text="Liste des actions  effectuer :").pack(anchor=W)
    self.w_actions = OrderedListBoxWithButtons(self._fr, self.object.actions, editbutton=1, height=4, width=40)
    self.w_actions.bind_new(self.new_action)
    self.w_actions.bind_edit(self.edit_action)
    self.w_actions.pack(fill=BOTH)
    self.w_after_pinni = BoolField(self._fr, "Appliquer aprs l'insertion du post dans le pinnipde", self.object.after_pinni)
    self.w_after_pinni.pack(fill=X, anchor=W)
    self.w_conditions = ConditionSetEditor(self._fr, self.object.conditions)
    self.w_conditions.pack(fill=X)
  
  def apply(self):
    self.object.actions = self.w_actions.getAll()
    self.object.after_pinni = self.w_after_pinni.get()
    self.w_conditions.apply()
    self.object.actif = 1
  
  def new_action(self):
    self.selected = None
    EventDialog(self, PyccEvent(action=ActionSetStyle()), self.extra_arg)
    if self.selected is not None:
      return self.selected.action
  
  def edit_action(self):
    action = self.w_actions.get(self.w_actions.selected)
    ActionDialog(self._fr, action, extra_arg=self.extra_arg)
    # self.w_actions.refresh()
    return 1
  

class PostProcessorEditor(CoinObjectEditor):
  
  def body(self):
    Label(self._fr, text="Edition des filtres du Processeur de Posts", font="Helvetica 9 bold", background=COLOR_TITLE).pack(fill=X)
    self.fr = Frame(self._fr)
    self.w_flist = OrderedListBoxWithButtons(self.fr, self.object.get(), editbutton=1, height=12, width=22)
    self.w_flist.bind_new(self.new_filter)
    self.w_flist.bind_edit(self.edit_filter)
    self.w_flist.bind_select(self.select_filter)
    self.w_flist.bind_delete(self.delete_filter)
    self.w_flist.pack(side=LEFT, fill=Y, anchor=NW)
    self.w_filters = [FilterEditor(self.fr, filter, extra_arg=self.extra_arg) for filter in self.object.get()]
    self.cur_feditor = None
    self.fr.pack(anchor=NW)
  
  def apply(self):
    for feditor in self.w_filters:
      feditor.apply()
    self.object.set(self.w_flist.getAll())
  
  def get_widget(self, filter=None):
    if filter is None:
      if self.w_flist.selected is None:
        return -1, None
      filter = self.w_flist.get(self.w_flist.selected)
    fwidget = [(idx, f) for idx, f in enumerate(self.w_filters) if f.object.name == filter.name]
    if len(fwidget) > 0:
      return fwidget[0]
    return -1, None
  
  def select_filter(self):
    if self.cur_feditor is not None:
      self.cur_feditor.pack_forget()
    idx, self.cur_feditor = self.get_widget()
    if self.cur_feditor is not None:
      self.cur_feditor.pack(padx=5, side=RIGHT, fill=BOTH)
  
  def delete_filter(self):
    idx, todelete = self.get_widget()
    if todelete is not None:
      del self.w_filters[idx]
    return 1
  
  def check_name(self, fname):
    return fname not in [f.name for f in self.w_flist.getAll()]
  
  def new_filter(self):
    res = tkSimpleDialog.askstring("Nouveau filtre", \
            "Entrez un nom pour le nouveau filtre :", initialvalue="Mon filtre", parent=self)
    if res is None:
      return
    if not self.check_name(res):
      tkMessageBox.showwarning("Nouveau filtre", "Un filtre de ce nom existe dj !", parent=self)
      return
    newfilter = Filter(res, actif=0)
    self.w_filters.append(FilterEditor(self.fr, newfilter, extra_arg=self.extra_arg))
    return newfilter
  
  def edit_filter(self):
    idx, torename = self.get_widget()
    res = tkSimpleDialog.askstring("Renommer un filtre", \
            "Entrez le nouveau nom du filtre :", initialvalue=torename.object.name, parent=self)
    if res is None or res == torename.object.name:
      return
    if not self.check_name(res):
      tkMessageBox.showwarning("Renommer un filtre", "Un autre filtre de ce nom existe dj !", parent=self)
      return
    torename.object.name = res
    self.w_flist.refresh()
