# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module forums
##   Dfinition du module de bouchot Forums :
##   monitoring des forums de linuxfr
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import xml.dom.minidom

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return Forums(parent, name)

class ForumsOptions(OptionSet):
  """ Options du module Forums
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "Forums"),
      'short_name':   Option("Nom court (pour la diode)", "F"),
      'backend':      Option("URL du fichier de backend", "/forums/"),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':     Option("Encoding du backend", "iso-8859-15"),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'backend', 'check_delay', 'max_history', 'read_at_exp', 'docked' ],
          "Parsing": [ 'tags_encoded', 'display_img', 'encoding' ]
        } )
    }


class ForumItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read     = 0   # Flag lu/nonlu
    self.expanded = 0   # Flag description ouverte/ferme
    self.next = None    # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def __str__(self):
    return repr(self.attributes['title'])
  
  def getTitle(self):
    return self.attributes['title']
  
  def getLink(self):
    return self.attributes['link']+self.attributes['groupid']+"/"+self.id+".html"
  
  def getGroupLink(self):
    return self.attributes['link']+self.attributes['groupid']+"/"
  
  def getDate(self):
    return self.attributes['date']
  
  def getAuthor(self):
    return self.attributes['author']
  
  def getGroup(self):
    return self.attributes['group']
  
  def getGroupId(self):
    return self.attributes['groupid']
  
  def getDescr(self):
    try:
      return self.attributes['description']
    except:
      return ''
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class ForumsViewer(GenericViewer):
  
  def insertItem(self, item):
    self.addFoldSign(item)
    self.list.config(state=NORMAL)
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "groupid%s" % item.getGroupId()
    self.list.insert(INSERT, item.getGroup(), t)
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getGroupLink())
    self.list.insert(INSERT, " : ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    self.module.bouchot.pycc.addUrlInList(item.getLink(), item.getDate()+"@"+self.module.name)
    if item.expanded:
      msg, tags = self.module.parseDescr(item.getDescr())
      # self.module.bouchot.pycc.addUrls([tag[4:] for tag in tags if tag[:4]=="url_"], item.getDate()+"@"+self.module.name)
      try:
        msg = toUnicode(msg, self.forum.options.val('encoding'))
      except:
        pass
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      self.addTags(tags, start)
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)


class Forums(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
  
  def build_viewer(self, parent):
    self.viewer = ForumsViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = ForumsOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def parse(self, file):
    tmplist = []
    text = file.read()
    encoding = "iso-8859-15"
    text = toUnicode(text, encoding)
    self.title = "DLFP : Forums..."
    start = text.find('<div class="titlediv">')
    end = 0
    while start > end:
      attrs = {}
      start = text.find('<a href="', start)
      end = text.find('/">', start)
      if end < start: break
      attrs['groupid'] = text[start+9:end]
      start = text.find('</a>', end)
      if start < end: break
      attrs['group'] = text[end+3:start].replace("\n", " ").strip()
      end = text.find('</h1>', start)
      if end < start: break
      attrs['title'] = text[start+7:end].replace("\n", " ").strip()
      start = text.find('/">', end)
      if start < end: break
      end = text.find('</a>', start)
      if end < start: break
      attrs['author'] = text[start+3:end].replace("\n", " ").strip()
      start = text.find('\n', end)
      if start < end: break
      end = text.find('<script', start)
      if end < start: break
      attrs['date'] = text[start+1:end].replace("\n", " ").strip()
      start = text.find(')</script>', end)
      if start < end: break
      attrs['id'] = text[end+44:start]
      start = text.find('<div class="bodydiv">', end)
      if start < end: break
      end = text.find('<p>', start)
      if end < start: break
      attrs['description'] = text[start+22:end].replace("\n", " ").strip()
      attrs['link'] = "http://%s%s" % (self.bouchot.options.val('http_server'), self.options.val('backend'))
      newitem = ForumItem(attrs)
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
      start = text.find('<div class="titlediv">', end)
    tmplist.reverse()
    return tmplist
  
  def process(self, items=[]):
    for forumitem in items:
      if self.firstId is None:
        self.firstId = forumitem.id
      if forumitem.id not in self.objects.keys():
        log.debug("%s: nouveau forumitem: %s" % (self.name, forumitem.id))
        self.objects[forumitem.id] = forumitem
        if self.lastId is not None:
          self.objects[self.lastId].next = forumitem
        self.lastId = forumitem.id
        if self.viewer_open:
          self.viewer.insertItem(forumitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    if not item.read:
      try:
        f = open(os.path.join(self.config_path, "read"), 'a')
        f.write(item.id+"\n")
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, id):
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      readlist = f.read().split("\n")
      f.close()
    except:
      log.warning("%s: Lecture dans le fichier 'read' impossible" % self.name)
      return 0
    return id in readlist
  
  def parseDescr(self, text):
    if self.options.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, self.options.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = getAttr("src", text[start:end])
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=self.options.val('encoding'))
    return (msg, tags)
  