# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module modules.module (ahah bis)
##   Classe-m�e pour tous les modules.
## ----------------------------------------------------------------------

import os, os.path
from urllib2 import HTTPError, URLError

import logging
log = logging.getLogger('pycc')

import sys, os.path
sys.path.append('..')

from Tkinter import *

from constants import *
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import CoinTopLevel
from coinlib.threads import GenericThread
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import getAttr, toString

class ModuleOptions(OptionSet):
  
  def setObjects(self):
    self._objects = {
      'descr':       Option("Description (nom pour affichage)", "Generic Module"),
      'short_name':  Option("Nom court (pour la diode)", "GM"),
      'backend':     Option("URL backend", "/"),
      'check_delay': Option("Rafrachir toutes les (en secondes)", 600, TYPE_INT),
      'encoding':    Option("Encoding du backend", "iso-8859-15"),
      'read_at_exp': Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'display_img': Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'docked':      Option("Docker dans le pinni", 0, TYPE_BOOL),
      'toedit':      Option("Editables", {"Gnral": ['backend']})
    }
  

class ModuleItem(object):
  """ Interface minimale pour un lment de module """
  
  def __init__(self, id=None, title="", link="", date=(1970,1,1,1,0,0,1,0,0), author="", descr="", categ=""):
    self.title    = title   # Titre
    self.link     = link    # URL
    self.intdate  = date    # Date interne, ci-dessous formate
    self.date = u"%4d/%02d/%02d %02d:%02d:%02d" % date[:6]
    self.author   = author  # Auteur
    self.descr    = descr   # Contenu
    self.categ    = categ   # Catgorie
    self.read     = 0       # Flag lu/nonlu
    self.expanded = 0       # Flag description ouverte/ferme
    self.next     = None    # Pointeur sur le suivant
    if id is not None:
      self.id = id
    else:
      self.computeId()
  
  def __cmp__(self, other):
    if self.read == other.read:
      if self.intdate > other.intdate:
        return 1
      if self.intdate < other.intdate:
        return -1
      return 0
    if self.read:
      return -1
    return 1
  
  def getLink(self):
    return self.link
  
  def getTitle(self):
    if self.title:
      return self.title
    return u"(pas de titre)"
  
  def computeId(self):
    self.id = md5.new(repr(self.title+self.date+self.author+self.descr)).hexdigest()

  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class Module(object):
  
  def __init__(self, bouchot, name):
    self.bouchot = bouchot   # Bouchot d'appartenance
    self.name = "%s:%s" % (bouchot.name, name)
    self.config_path = os.path.join(self.bouchot.config_path, name)
    self.loaded = 0
    self.load_config()
  
  def create_config(self):
    os.mkdir(self.config_path)
  
  def load_config(self):
    log.debug("Chargement config module %s" % self.name)
    self.options = ModuleOptions(path=self.config_path)  # Override in derivated classes
  
  def save_config(self):
    log.debug("Sauvegarde config module %s" % self.name)
    self.options.save()
  
  def apply_config(self):
    pass # Override in derivated classes
  
  def config_frame(self, parent):
    return OptionsEditor(parent, self.options, extra_arg=self.bouchot)
  
  def build_viewer(self, parent):
    self.viewer = GenericViewer(parent, self)  # Override in derivated classes
  
  def apply_styles(self):
    pass # Override in derivated classes
  
  def add_led(self):
    log.debug("Ajout de la diode pour %s" % self.name)
    try:
      shname = self.options.val('short_name')
    except PyccError:
      shname = self.name[:1]
    self.led = Led(self.bouchot.ledbar, shname)
    self.led.pack(side='left')
    for i in range(1,4):
      self.led._led.bind("<ButtonRelease-%s>"%i, lambda e, s=self, act=i: s.led_click(act))
    if self.bouchot.pycc.options.val('emul_3but'):
      self.led._led.bind("<Control-ButtonRelease-1>", lambda e, s=self: s.led_click(2))
  
  def initialize(self):
    log.debug("Initialisation du module %s" % self.name)
    self.objects = {}
    try:
      delay = self.options.val('check_delay')
    except PyccError:
      delay = 300 # refresh  5 minutes par dfaut
    self.thread = GenericThread(self.name, delay)
    self.thread.process = self.action
    self.thread._initialize(delay)
    self.add_led()
    self.viewer = None
    self.viewer_open = 0
    self.lastModif = ''
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      self._readlist = f.read().split("\n")
      f.close()
    except:
      log.info("%s: Lecture dans le fichier 'read' impossible" % self.name)
      self._readlist = []
    self.init()  # Initialisation spcifique
    self.loaded = 1
  
  def init(self):
    pass # Override
  
  def openViewer(self):
    if self.viewer_open:
      self.viewer.focus_set()
      return
    if self.viewer is None:
      if self.options.val('docked'):
        parent = self.bouchot.pw.add("%s:view" % self.name, min=.1, size=.4)
        self.bouchot.pw.updatelayout()
      else:
        parent = CoinTopLevel(self.bouchot.root, self.options.val('descr'))
        parent.protocol("WM_DELETE_WINDOW", self.closeViewer)
      self.build_viewer(parent)
    self.show()
  
  def closeViewer(self):
    if self._readlist:
      try:
        f = open(os.path.join(self.config_path, "read"), 'w')
        f.write("\n".join(self._readlist[-1000:]))
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    self.hide()
    self.viewer.root.destroy()
    self.viewer = None
  
  def show(self):
    self.viewer.pack(fill=BOTH, expand=1)
    self.viewer_open = 1
  
  def hide(self):
    if self.options.val('docked'):
      self.bouchot.pw.delete("%s:view" % self.name)
      self.bouchot.pw.updatelayout()
    self.viewer_open = 0
  
  def toggleViewMode(self):
    self.closeViewer()
    actual = self.options.val('docked')
    self.options.set('docked', not actual)
    self.openViewer()
  
  def start(self):
    log.info("Dmarrage %s" % self.name)
    self.thread.start()
  
  def setState(self, state, msg=''):
    self.state = state
    if state == STATE_OK:
      self.led.clear()
    else:
      self.led.setState(state, msg)
      if state != STATE_HTTP:
        self.bouchot.setState(state)
  
  def get_http_params(self):
    # Override
    # Retourne: ua, cookie et data
    return ('', '', None)
  
  def unchanged(self, conn):
    last = conn.info().getheader('Last-Modified')
    if last and self.lastModif == last:
      return 1
    self.lastModif = last
    return 0
  
  def getEncoding(self, line):
    if line.find("<?xml") >= 0:
      try:
        encoding = getAttr("encoding", line)
        self.options.set("encoding", encoding)
        log.info("%s: Encoding dtact dans le backend: %s" % (self.name, encoding))
        return encoding
      except:
        pass
    encoding = self.options.val('encoding')
    log.info("%s: Pas d'encoding dans le backend, valeur par dfaut: %s" % (self.name, encoding))
    return encoding
  
  def parse(self, file):
    log.debug("%s: mthode parse gnrique" % self.name)
    print file.read()
    return []
  
  def action(self):
    self.setState(STATE_HTTP)
    server = self.bouchot.options.val('http_server')
    url = self.options.val('backend')
    ua, cookie, data = self.get_http_params()
    headers = {}
    if self.lastModif:
      headers['If-Modified-Since'] = self.lastModif
    self.error = ""
    try:
      file = open_url(server, url, self.bouchot.pycc.options, ua, cookie, data, headers)
    except HTTPError, value:
      if value.code == 302 or value.code == 304:
        self.setState(STATE_OK)
        log.debug("%s - %s" % (url, value))
      else:
        self.error = str(value)
        self.setState(STATE_ERROR, self.error)
        log.error("Erreur HTTP sur %s - %s:\n    %s" % (self.name, url, self.error))
      return
    except URLError, value:
      self.error = str(value.reason)
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur en essayant de rapatrier %s:\n    %s" % (self.name, url, self.error))
      return
    except:
      self.error = "Y'a eu un truc zarb sur %s" % url
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur http inconnue sur %s" % (self.name, url))
      return
    if self.unchanged(file):
      file.close()
      log.debug("%s n'a pas chang" % url)
      return
    object_list = []
    try:
      object_list = self.parse(file)
    except:
      self.error = "Ca a plant bizarrement"
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur inconnue en parsant %s" % (self.name, url))
    else:
      if self.error:
        self.setState(STATE_WARNING, self.error)
        log.warning("Erreur de parsing sur %s:\n    %s" % (self.name, self.error))
      else:
        self.setState(STATE_OK)
    file.close()
    if object_list:
      self.process(object_list)  # Ce qu'on fait des objets parss
  
  def process(self, object_list=[]):
    log.debug("Je fais le process de %s" % self.name)
    # Abstract --  implmenter dans les modules drivs
  
  def isRead(self, itemid):
    return itemid in self._readlist
  
  def markRead(self, item, update_viewer=1):
    #print "DBG: entree dans module.markRead", item.id
    if not self.isRead(item.id):
      self._readlist.append(item.id)
      item.markRead()
      if self.viewer_open and update_viewer:
        self.viewer.markRead(item)
    self.updateLed()
  
  def markUnread(self, item, update_viewer=1):
    #print "DBG: entree dans module.markUnread", item.id
    if self.isRead(item.id):
      del self._readlist[self._readlist.index(item.id)]
      item.markUnread()
      if self.viewer_open and update_viewer:
        self.viewer.markUnread(item)
    self.updateLed()
  
  def led_click(self, action):
    if action == 1:
      if self.viewer_open:
        self.closeViewer()
      else:
        self.openViewer()
    elif action == 2:
      self.led.showMsg(self.bouchot.pw.interior(), "Dernire erreur sur %s" % self.name)
    else:
      self.thread.restart()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def unload(self):
    pass  # Override
  
  def quit(self):
    if not self.loaded:
      log.debug("%s.quit(): module dcharg" % self.name)
      return
    if self.viewer is not None:
      self.closeViewer()
    self.thread.stop()
    self.led.destroy()
    self.unload()
    self.loaded = 0
  

class GenericViewer(Frame):
  
  def __init__(self, parent, module):
    self.root = parent
    self.module = module
    self.styles = self.module.bouchot.styles
    Frame.__init__(self, parent)
    self.build_interface()
    self.update()
  
  def _build_topframe(self):
    fr = Frame(self)
    self.butmenu = Label(fr, text="Actions >", relief=FLAT, border=2)
    self.butmenu.bind("<Enter>", lambda e, s=self: s.butmenu.config(relief=RAISED))
    self.butmenu.bind("<Leave>", lambda e, s=self: s.butmenu.config(relief=FLAT))
    self.butmenu.bind("<1>", self.show_menu)
    self.butmenu.pack(side=LEFT)
    self.butclose = Label(fr, text=" X ", relief=FLAT, border=2)
    self.butclose.bind("<Enter>", lambda e, s=self: s.butclose.config(relief=RAISED))
    self.butclose.bind("<Leave>", lambda e, s=self: s.butclose.config(relief=FLAT))
    self.butclose.bind("<1>", self.quit)
    self.butclose.pack(side=RIGHT)
    self.label = Label(fr, font="Helvetica 10 bold", background=COLOR_TITLE)
    self.label.pack(side=LEFT, fill=X, expand=1)
    fr.pack(fill=X)
  
  def _build_ctxmenu(self):
    self.menu = Menu(self.root, tearoff=0)
    self.build_menu()
    self.menu.bind("<Escape>", self.hide_menu)
    self.ascending = IntVar()
    self.ascending.set(1)
    self.menu.add_checkbutton(label="Les plus rcents en haut", command=self.toggleSort, variable=self.ascending)
    if self.module.options.val('docked'):
      btxt = "Dtacher"
    else:
      btxt = "Docker"
    self.menu.add_command(label=btxt, command=self.module.toggleViewMode)
    self.menu.add_command(label="Configurer module...", command=self.showConfig)
    self.menu.add_command(label="Fermer", command=self.quit)
  
  def _build_listview(self):
    self.list = ScrolledText(self, wrap=WORD)
    self.list.pack(side=TOP, fill=BOTH, expand=1)
    self.list.bind("<Enter>", lambda e, w=self: w.list.focus_set())
    self.list.bind("<Motion>", self.mouse_move)
    self.list.bind("<1>", self.mouse_click)
    self.list.bind("<2>", self.mouse_click2)
    self.list.bind("<3>", self.show_menu)
    self.list.bind("<Escape>", self.hide_menu)
  
  def build_menu(self):
    # A surcharger si besoin
    self.menu.add_command(label="Marquer tous lus", command=self.markAllRead)
    self.menu.add_command(label="Drouler les non lus", command=self.expandAllUnread)
    self.menu.add_command(label="Ouvrir les non lus", command=self.openAll)
    self.menu.add_command(label="Drouler tout", command=self.expandAll)
    self.menu.add_command(label="Enrouler tout", command=self.collapseAll)
  
  def build_interface(self):
    self._build_topframe()
    self._build_ctxmenu()
    self._build_listview()
  
  def show_menu(self, event):
    self.menu.post(event.x_root, event.y_root)
  
  def hide_menu(self, event=None, **kw):
    self.menu.unpost()
  
  def showConfig(self, event=None):
    self.module.bouchot.pycc.showConfig(self.module.name.replace(":", "_"))
  
  def update(self):
    """ Mise  jour du contenu de la fentre """
    self.setLbl(self.module.title)
    self.list.config(state=NORMAL)
    self.list.config(cursor="arrow")
    # Effacement de l'ancien contenu
    self.list.delete(0.0, END)
    # Configuration des styles
    self.apply_styles()
    item = None
    if len(self.module.objects.keys()) > 0:
      try:
        item = self.module.objects[self.module.firstId]
      except:
        pass
    # Insertion des items
    while item is not None:
      if self.ascending.get():
        self.list.mark_set(INSERT, 0.0)
      else:
        self.list.mark_set(INSERT, END)
      self.insertItem(item)
      item = item.next
    #else:
    #  self.list.insert(END, "Aucun lment trouv !")
    self.list.config(state=DISABLED)
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.module.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
  
  def toggleSort(self, event=None):
    cursort = self.ascending.get()
    self.update()
  
  def get_tags_id(self, event):
    log.debug("## Entering GenericViewer.get_tags_id")
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    cur_id = -1
    tres = []
    for t in tags:
      if t[:2] == 'id':
        cur_id = t[2:]
      tres.append(t)
    tres.sort()
    tres.reverse()
    return cur_id, tres
  
  def mouse_move(self, event):
    self.unhilight()
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    # id, tags = self.get_tags_id(event)
    for t in tags:
      if t[:4] == "url_" or t[:4] == "img_":
        self.module.bouchot.pycc.setStatus(t[4:])
        self.setStyleTag(t, "urlhilight")
        self.list.config(cursor="hand2")
  
  def mouse_click2(self, event):
    self.mouse_click(event, 2)
  
  def mouse_click(self, event, but=1):
    # tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    self.hide_menu()
    id, tags = self.get_tags_id(event)
    if id == -1:
      return
    try:
      item = self.module.objects[id]
    except:
      log.warning("%s: pas d'objet trouv puor l'id %s" % (self.name, id))
      return
    for t in tags:
      if t[:5] == "read_":
        self.toggleRead(item)
        return
      if t[:5] == "fold_":
        self.toggle(item)
        return
      if t[:4] == "url_":
        url = t[4:]
        if but == 1:
          self.module.bouchot.pycc.openBrowser(url)
          self.setStyleTag(t, "urlvisit")
          self.module.bouchot.pycc.setUrlRead(url)
        else:
          uurl = toUnicode(url)
          self.module.bouchot.pycc.palmi.palmi.inserter(uurl, None, 0)
          self.module.bouchot.pycc.palmi.tabselect('Message')
          self.module.bouchot.pycc.palmi.palmi.focus_set()
          self.module.bouchot.pycc.clipboard = uurl
      if t[:5] == "title" and but == 1:
        self.module.markRead(item)
      if t[:4] == "img_" and but != 1:
        url = t[4:]
        self.module.bouchot.pycc.getImageFromUrl(url)
        self.updateItem(item)
  
  def addTags(self, tags, start=None):
    if start is None:
      start = self.list.index("insert")
    for tag in tags.keys():
      for range in tags[tag]:
        self.list.tag_add(tag, "%s+%s chars" % (start, range[0]), "%s+%s chars" % (start, range[1]))
      if tag[:4] == 'url_':
        self.module.bouchot.pycc.addUrlInList(tag[4:], "@"+self.module.name)
        if self.module.bouchot.pycc.isUrlRead(tag[4:]):
          self.setStyleTag(tag, 'urlvisit')
    if tags.has_key('img'):
      if self.module.options.val('display_img'):
        myranges = tags['img']
        myranges.sort()
        myranges.reverse()
        for range in myranges:
          idx_start = "%s+%s chars" % (start, range[0])
          idx_end = "%s+%s chars" % (start, range[1])
          url = toString(self.list.get(idx_start, idx_end)[7:-1])
          imgname = md5.new(url).hexdigest()
          self.list.delete(idx_start, idx_end)
          try:
            if not self.module.bouchot.pycc.images.has_key(imgname):
              self.module.bouchot.pycc.images[imgname] = get_image(os.path.join(self.module.bouchot.pycc.config_path, "cache", imgname))
            self.list.image_create(idx_start, image=self.module.bouchot.pycc.images[imgname], align=CENTER)
            self.list.tag_add("url_%s" % url, idx_start, "%s+1c" % idx_start)
          except:
            log.warning("%s: Image '%s' introuvable ou fichier corrompu" % (self.module.name, imgname))
            self.list.insert(idx_start, "## Erreur ## Image: %s" % imgname, ("img", "img_%s" % url, "url_%s" % url))
      # self.list.tag_raise('a') # pour viter que img se retrouve dessus
  
  def setStyleTag(self, tag, style):
    tseq = self.list.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      try:
        self.list.tag_add(style, tseq[cpt], tseq[cpt+1])
      except:
        log.warning("Vautrage tk sur setStyleTag(%s, %s)" % (tag, style))
      cpt += 2
  
  def delStyleTag(self, tag, style):
    tseq = self.list.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      try:
        self.list.tag_remove(style, tseq[cpt], tseq[cpt+1])
      except:
        log.warning("Tk a plantai sur delStyleTag(%s, %s)" % (tag, style))
      cpt += 2
  
  def hilight(self, id):
    """ Mise en surbrillance d'un item """
    beg, end = self.list.tag_ranges("id%s" % id)
    self.list.tag_add("urlhilight", beg, end)
    self.list.config(cursor="hand2")
  
  def unhilight(self, event=None):
    """ Enlve les surbrillances """
    self.list.tag_remove("urlhilight", 1.0, END)
    self.list.config(cursor="arrow")
    self.module.bouchot.pycc.setStatus("")
  
  def openAll(self, event=None):
    for item in [i for i in self.module.objects.values() if not i.read]:
      self.openItem(item)
  
  def openItem(self, item):
    """ Ouvre un lien dans un browser """
    self.module.bouchot.pycc.openBrowser(item.getLink())
    self.module.markRead(item)
  
  def insertItem(self, item):
    pass # Methode abstraite
  
  def addReadSign(self, item):
    #print "DBG: entree dans genericViewer: addReadSign", item.id
    if item.read:
      sign = self.module.bouchot.pycc.images['read']
    else:
      sign = self.module.bouchot.pycc.images['unread']
    self.list.image_create(INSERT, image=sign, align=CENTER)
    self.list.tag_add("read_%s" % item.id, "insert-1c", "insert-0c")
    #self.list.tag_bind("read_%s" % item.id, "<ButtonRelease-1>", lambda e, w=self, i=item: w.toggleRead(i))
  
  def addFoldSign(self, item):
    #print "DBG: entree dans genericViewer: addFoldSign", item.id
    if item.expanded:
      #sign = self.module.bouchot.pycc.images['minus']
      sign = self.module.bouchot.pycc.images['unfolded']
    else:
      #sign = self.module.bouchot.pycc.images['plus']
      sign = self.module.bouchot.pycc.images['folded']
    self.list.image_create(INSERT, image=sign, align=CENTER)
    self.list.tag_add("fold_%s" % item.id, "insert-1c", "insert-0c")
    #self.list.tag_bind("fold_%s" % item.id, "<ButtonRelease-1>", lambda e, w=self, i=item: w.toggle(i))
  
  def updateItem(self, item):
    #print "DBG: entree dans genericViewer: updateItem", item.id
    self.list.config(state=NORMAL)
    itag = "id%s" % item.id
    try:
      pos = self.list.tag_ranges(itag)
      start = pos[0]
      end = pos[1]
    except:
      log.error("%s: tag '%s' inexistant - impossible de trouver les bornes" % (self.module.name, itag))
      return
    self.list.delete(start, end)
    self.list.mark_set(INSERT, start)
    self.insertItem(item)
    self.list.config(state=DISABLED)
  
  def setLbl(self, text):
    self.label.config(text=text)
  
  def toggleRead(self, item):
    #print "DBG: entree dans genericViewer.toggleRead", item.id
    if item.read:
      self.module.markUnread(item)
    else:
      self.module.markRead(item)
  
  def _updateReadSign(self, item):
    #print "DBG: entree dans genericViewer.updateReadSign", item.id
    # Met  jour la marque de read/unread
    self.list.config(state=NORMAL)
    rtag = "read_%s" % item.id
    try:
      pos = self.list.tag_ranges(rtag)
      start = pos[0]
      end = pos[1]
    except:
      log.error("%s: tag '%s' inexistant - impossible de trouver les bornes" % (self.module.name, rtag))
      return
    self.list.mark_set(INSERT, end)
    self.addReadSign(item)
    self.list.delete(start)
    self.list.config(state=DISABLED)
  
  def markUnread(self, item):
    #print "DBG: entree dans genericViewer.markUnread", item.id
    self._updateReadSign(item)
    self.delStyleTag("url_"+item.getLink(), "urlvisit")
  
  def markRead(self, item):
    #print "DBG: entree dans genericViewer.markRead", item.id
    self._updateReadSign(item)
    self.markUrlRead(item)
  
  def markUrlRead(self, item):
    #print "DBG: entree dans genericViewer: markUrlRead", item.id
    # Marque les liens associs comme lus
    self.setStyleTag("url_"+item.getLink(), "urlvisit")
    self.module.bouchot.pycc.setUrlRead(item.getLink())
  
  def markAllRead(self, event=None):
    for item in self.module.objects.values():
      self.module.markRead(item)
  
  def expandAllUnread(self, event=None):
    for item in self.module.objects.values():
      item.expanded = not item.read
      if self.module.options.val('read_at_exp'):
        self.module.markRead(item, 0)
      self.updateItem(item)
  
  def expandAll(self, event=None):
    for item in self.module.objects.values():
      if self.module.options.val('read_at_exp'):
        self.module.markRead(item, 0)
      item.expanded = 1
    self.update()
  
  def collapseAll(self, event=None):
    for item in self.module.objects.values():
      item.expanded = 0
    self.update()
  
  def showItem(self, item):
    item.expanded = 1
    if self.module.options.val('read_at_exp'):
      self.module.markRead(item, 0)
    self.updateItem(item)

  def hideItem(self, item):
    item.expanded = 0
    self.updateItem(item)
  
  def toggle(self, item):
    if item.expanded:
      self.hideItem(item)
    else:
      self.showItem(item)
    self.list.see(self.list.tag_ranges('id%s' % item.id)[0])
  
  def quit(self, event=None):
    self.module.closeViewer()
