# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module multirss
##   Dfinition du module de bouchot MultiRSS :
##   monitoring de fils rss multiples dans une mme fentre
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import feedparser
import tkMessageBox

from constants import *
from modules.module import *
from coinlib.persistants import XMLizable, OptionSet, Option, RSSItemOptions
from coinlib.utils import *
from coinlib.parsers import *
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import OptionsDialog

def build_module(parent, name):
  return MultiRSS(parent, name)

class MultiRSSOptions(OptionSet):
  """ Options du module MultiRSS
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "MultiRSS"),
      'short_name':   Option("Nom court (pour la diode)", "MultiRSS"),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'check_delay':  Option("Rafrachir toutes les (en secondes)", 600, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 200, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'rss_list':     Option("Liste des fils RSS", [RSSItemOptions()], TYPE_LIST),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'check_delay', 'max_history', 'read_at_exp', 'docked', 'display_img' ],
          "Fils RSS":[ 'rss_list' ]
        } )
    }


class MultiRSSEditor(OptionsEditor):
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = OrderedListBoxWithButtons(fr, option.get(), 1, width=40, height=10)
    self.widgets[name].pack(fill=X)
    self.widgets[name].bind_new(self.new)
    self.widgets[name].bind_edit(self.edit)
  
  def new(self):
    self.canceled = 1
    newfeed = RSSItemOptions()
    OptionsDialog("Nouveau fil RSS", self, newfeed, self.extra_arg)
    if not self.canceled:
      return newfeed
  
  def edit(self):
    rssitem = self.widgets['rss_list'].get(self.widgets['rss_list'].selected)
    OptionsDialog("Proprits du fil RSS", self, rssitem, self.extra_arg)
    # self.w_actions.refresh()
    return 1
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'rss_list':
        self.object.set(opt, widget.getAll())
      else:
        self.object.set(opt, widget.get())
    self.object.save()


class MultiRSSItem(ModuleItem):
  
  def __init__(self, id=None, title="", link="", date=(1970,1,1,1,0,0,1,0,0), author="", descr="", categ=""):
    ModuleItem.__init__(self, id, title, link, date, author, descr, categ)
    self.feedopts = None    # Pointeur sur les options du feed
  

class MultiRSSViewer(GenericViewer):

  def __init__(self, parent, module):
    self.cur_opts = RSSItemOptions()
    GenericViewer.__init__(self, parent, module)
  
  def build_interface(self):
    GenericViewer.build_interface(self)
    fr = Frame(self)
    self.feedname = TextField(fr, "Nom:", width=12)
    self.feedname.pack(side=LEFT)
    self.feedurl = TextField(fr, "URL:", width=42)
    self.feedurl.pack(side=LEFT, fill=X)
    Button(fr, text="Ajouter", border=1, command=self.addRSS).pack(side=LEFT, fill=X)
    Button(fr, text="Options", border=1, command=self.showRSSOpts).pack(side=LEFT, fill=X)
    fr.pack(side=TOP, fill=X)
  
  def show_menu(self, event):
    """
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      cur_rss = [rss for rss in self.multirss.options.val('rss_list') if rss.val('name') == t]
      if cur_rss:
        self.menu.add_command("Config du rss courant")
    # AFR: c'est juste l'ide
    """
    self.menu.post(event.x_root, event.y_root)
  
  def showRSSOpts(self):
    self.cur_opts.set('name', self.feedname.get())
    self.cur_opts.set('backend', self.feedurl.get())
    OptionsDialog("Proprits du fil RSS", self, self.cur_opts, self.module.bouchot.pycc)
  
  def addRSS(self):
    feedname = self.feedname.get()
    feedurl = self.feedurl.get()
    if not feedname or not feedurl:
      return
    rsslist = self.module.options.val('rss_list')
    if feedname in [ rss.val('name') for rss in rsslist ]:
      tkMessageBox.showwarning('MultiRSS', "Un fil de nom %s existe dj" % feedname, parent=self)
      return
    if feedurl in [ rss.val('backend') for rss in rsslist ]:
      tkMessageBox.showwarning('MultiRSS', "Un fil d'url %s existe dj" % feedurl, parent=self)
      return
    newrss = self.cur_opts
    self.cur_opts = RSSItemOptions()
    newrss.set('name', feedname)
    newrss.set('backend', feedurl)
    self.module.options.set('rss_list', rsslist+[newrss])
    self.module.options.save()
    self.module.thread.restart()
    self.apply_styles()
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.module.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
    for rssopt in self.module.options.val('rss_list'):
      self.list.tag_config(rssopt.val('name'), background=rssopt.val('color'))
      self.list.tag_raise(rssopt.val('name'))
  
  def insertItem(self, item):
    #print "DBG: entry in multirss.insertItem"
    self.list.config(state=NORMAL)
    self.addReadSign(item)
    #print "DBG: read sign added"
    self.addFoldSign(item)
    #print "DBG: fold sign added"
    orig_pos = self.list.index("insert-2c")
    if orig_pos is None:
      log.warning("Tk suxe, il dit que la position de INSERT est None alors que c'est mme pas vrai")
      orig_pos = self.list.index("insert-2c")
    self.list.insert(INSERT, "[%s]" % item.feedopts.val('name'), item.feedopts.val('name'))
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.date, "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.author, "login")
    self.list.insert(INSERT, " ")
    if item.categ:
      self.list.insert(INSERT, "["+item.categ+"]", "info")
      self.list.insert(INSERT, " ")
    t = "title%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    self.module.bouchot.pycc.addUrlInList(item.getLink(), item.date+"@"+self.module.name+":"+item.feedopts.val('name'))
    #print "DBG: headline parameters added"
    if item.expanded:
      text = toUnicode(item.descr, item.feedopts.val('encoding'))
      #print "DBG: before parseDescr"
      msg, tags = self.module.parseDescr(text, item.feedopts)
      #print "DBG: after parseDescr"
      # self.module.bouchot.pycc.addUrls([tag[4:] for tag in tags if tag[:4]=="url_"], item.getDate()+"@"+self.module.name+":"+item.feedopts.val('name'))
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      end_pos = self.list.index("insert-0c")
      self.addTags(tags, start)
    else:
      end_pos = self.list.index("insert-0c")
    i = "id%s" % item.id
    try:
      self.list.tag_add(i, orig_pos, end_pos)
    except:
      log.warning("Tk suxor, j'ai pas pu mettre le tag sur un lment rss, a va tre dur de le lire.")
    #print "DBG: id tag added"
    # print "j'ai mis un tag sur", orig_pos, end_pos
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
      #print "DBG: item marked as read"
  

class MultiRSS(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
    self.openViewer()
    self.apply_styles()
  
  def config_frame(self, master):
    return MultiRSSEditor(master, self.options, self.bouchot.pycc)
  
  def build_viewer(self, parent):
    self.viewer = MultiRSSViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = MultiRSSOptions(path=self.config_path)
  
  def action(self):
    for rssitem in self.options.val('rss_list'):
      self.error = ""
      rssname = rssitem.val('name')
      rssurl = rssitem.val('backend')
      self.led.setLabel("["+rssname+"]")
      if rssurl[:4].lower() == 'stop':
        continue
      self.setState(STATE_HTTP)
      if rssurl[:7] == 'http://':
        rssurl = rssurl[7:]
      pos = rssurl.find('/')
      server = rssurl[:pos]
      url = rssurl[pos:]
      ua, cookie, data = self.get_http_params()
      headers = {}
      try:
        file = open_url(server, url, self.bouchot.pycc.options, ua, cookie, data, headers)
      except HTTPError, value:
        if value.code == 302 or value.code == 304:
          self.setState(STATE_OK)
          log.debug("%s - %s" % (url, value))
        else:
          self.error = "[%s] %s" % (rssname, str(value))
          self.setState(STATE_ERROR, self.error)
          log.error("Erreur HTTP sur %s:%s - %s:\n    %s" % (self.name, rssname, rssurl, str(value)))
      except URLError, value:
        self.error = "[%s] %s" % (rssname, str(value.reason))
        self.setState(STATE_ERROR, self.error)
        log.error("%s:%s Erreur en essayant de rapatrier %s:\n    %s" % (self.name, rssname, rssurl, str(value.reason)))
      except:
        self.error = "[%s] Y'a eu un truc zarb sur %s" % (rssname, rssurl)
        self.setState(STATE_ERROR, self.error)
        log.error("%s:%s Erreur http inconnue sur %s" % (self.name, rssname, rssurl))
      object_list = []
      if not self.error:
        try:
          object_list = self.parse(file, rssitem)
        except:
          self.error = "[fil %s] Ca a plant bizarrement" % rssname
          self.setState(STATE_ERROR, self.error)
          log.error("%s:%s Erreur inconnue en parsant %s" % (self.name, rssname, rssurl))
        else:
          if self.error:
            self.setState(STATE_WARNING, self.error)
            log.warning("Erreur de parsing sur %s:%s\n    %s" % (self.name, rssname, self.error))
          else:
            self.setState(STATE_OK)
        file.close()
      if object_list:
        self.process(object_list)  # Ce qu'on fait des objets parss
    self.led.setLabel(self.options.val('short_name'))
    
  
  def parse(self, file, rssitem):
    tmplist = []
    text = file.read()
    data = feedparser.parse(text)
    rssitem.set('encoding', data.encoding)
    title = data.feed.title
    if not rssitem.val('keep_cr'):
      title = title.replace('\n', ' ')
    rssitem.set('descr', title)
    for elem in data.entries:
      id = None
      title = elem.title
      if not rssitem.val('keep_cr'):
        title = title.replace('\n', ' ')
      link = elem.link
      try:
        date = elem.date_parsed
      except:
        date = (2000,1,1,0,0,0,1,0,0)
      try:
        descr = u'\n'.join(c['value'] for c in elem.content)
      except:
        descr = ""
      if not descr:
        try:
          descr = elem.description
        except:
          pass
      if not rssitem.val('keep_cr'):
        descr = descr.replace('\n', ' ')
      try:
        author = elem.author
      except:
        author = ""
      try:
        categ = elem.category
      except:
        categ = ""
      newitem = MultiRSSItem(id, title, link, date, author, descr, categ)
      newitem.feedopts = rssitem
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
    tmplist.reverse()
    tmplist.sort()
    return tmplist
  
  def process(self, items=[]):
    for rssitem in items:
      if self.firstId is None:
        self.firstId = rssitem.id
      if rssitem.id not in self.objects.keys():
        log.debug("%s: nouvel rssitem: %s" % (self.name, rssitem.id))
        self.objects[rssitem.id] = rssitem
        if self.lastId is not None:
          self.objects[self.lastId].next = rssitem
        self.lastId = rssitem.id
        if self.viewer_open:
          if self.viewer.ascending.get():
            self.viewer.list.mark_set(INSERT, 0.0)
          else:
            self.viewer.list.mark_set(INSERT, END)
          self.viewer.insertItem(rssitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def parseDescr(self, text, rssopt):
    if rssopt.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    #print "DBG: before html.pre_parse"
    text = htmlParser.pre_parse(text, slip_style, rssopt.val('encoding'))
    #print "DBG: pre parsing done"
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = toString(get_url_from_base(rssopt.val('backend'), getAttr("src", text[start:end])))
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        #print "DBG: before getImage:", url
        self.bouchot.pycc.getImageWithCache(url)
    #print "DBG: before html_parse"
    msg, tags = htmlParser.html_parse(text, encoding=rssopt.val('encoding'), base_href=rssopt.val('backend'))
    #print "DBG: descr html parsed"
    return (msg, tags)
  