# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module rssreader
##   Dfinition du module de bouchot RSSReader :
##   monitoring d'un feed rss (pour les news, etc.)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import feedparser

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return RSSReader(parent, name)

import re
regexp_newsid = re.compile("^.*/([0-9]+)/([0-9]+)/([0-9]+)/([^/,.]+)[,.][^/]*$")

class RSSReaderOptions(OptionSet):
  """ Options du module RSSReader
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "RSSReader"),
      'short_name':   Option("Nom court (pour la diode)", "N"),
      'backend':      Option("URL du fichier de backend", ""),
      'get_with_cookie': Option("Envoyer le(s) cookie(s) dans les requtes backend", 0, TYPE_BOOL),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':     Option("Encoding du backend", "iso-8859-15"),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'slip_crade':   Option("Slip pas propre", 0, TYPE_BOOL),
      'keep_cr':      Option("Conserver les retours  la ligne", 1, TYPE_BOOL),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'find_author':  Option("Trouver l'auteur dans l'url (journaux dlfp)", 0, TYPE_BOOL),
      'find_date':    Option("Trouver la date dans l'url (news dlfp)", 0, TYPE_BOOL),
      'find_id':      Option("Trouver l'ID dans l'url", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'backend', 'get_with_cookie', 'check_delay', 'max_history', 'read_at_exp', 'docked' ],
          "Parsing": [ 'tags_encoded', 'keep_cr', 'slip_crade', 'find_date', 'find_author', 'find_id', 'display_img', 'encoding' ]
        } )
    }


class RSSItem(ModuleItem):
  
  def getDate(self):
    return u"%02d/%02d/%4d %02d:%02d:%02d" % (self.intdate[2], self.intdate[1], self.intdate[0], self.intdate[3:6])
  

class RSSViewer(GenericViewer):

  def insertItem(self, item):
    self.list.config(state=NORMAL)
    self.addReadSign(item)
    self.addFoldSign(item)
    orig_pos = self.list.index("insert-2c")
    if orig_pos is None:
      log.warning("Tk suxe, il dit que la position de INSERT est None alors que c'est mme pas vrai")
      orig_pos = self.list.index("insert-2c")
    self.list.insert(INSERT, item.date, "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.author, "login")
    self.list.insert(INSERT, " ")
    if item.categ:
      self.list.insert(INSERT, "["+item.categ+"]", "info")
      self.list.insert(INSERT, " ")
    t = "title%s" % item.id
    self.list.insert(INSERT, item.title, t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.link)
    self.module.bouchot.pycc.addUrlInList(item.link, item.date+"@"+self.module.name)
    if item.expanded:
      text = toUnicode(item.descr, self.module.options.val('encoding'))
      msg, tags = self.module.parseDescr(text)
      # self.module.bouchot.pycc.addUrls([tag[4:] for tag in tags if tag[:4]=="url_"], item.getDate()+"@"+self.module.name)
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      end_pos = self.list.index("insert-0c")
      self.addTags(tags, start)
    else:
      end_pos = self.list.index("insert-0c")
    i = "id%s" % item.id
    try:
      self.list.tag_add(i, orig_pos, end_pos)
    except:
      log.warning("Tk suxor, j'ai pas pu mettre le tag sur un lment rss, a va tre dur de le lire.")
    # print "ONE tag sur", orig_pos, end_pos
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  

class RSSReader(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
  
  def build_viewer(self, parent):
    self.viewer = RSSViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = RSSReaderOptions(path=self.config_path)
  
  def get_http_params(self):
    cookie = ''
    if self.options.val('get_with_cookie'):
      idname = self.bouchot.options.val('default_id')
      id = self.bouchot.pycc.identities.get(idname)
      try:
        cookie = id.cookies[self.bouchot.name]
      except:
        pass
    return ('', cookie, None)
  
  def parse(self, file):
    tmplist = []
    text = file.read()
    data = feedparser.parse(text)
    if data.feed.title:
      self.title = data.feed.title
      if not self.options.val('keep_cr'):
        self.title = self.title.replace('\n', ' ')
    self.options.set('encoding', data.encoding)
    for elem in data.entries:
      id = None
      title = elem.title
      if not self.options.val('keep_cr'):
        title = title.replace('\n', ' ')
      link = elem.link
      try:
        date = elem.date_parsed
      except:
        date = (2000,1,1,0,0,0,1,0,0)
      try:
        descr = u'\n'.join(c['value'] for c in elem.content)
      except:
        descr = elem.description
      if not self.options.val('keep_cr'):
        descr = descr.replace('\n', ' ')
      try:
        author = elem.author
      except:
        author = ""
      try:
        categ = elem.category
      except:
        categ = ""
      if self.options.val('find_date'):
        newsid = regexp_newsid.match(link)
        if self.options.val('find_id'):
          id = newsid.groups()[3]
        date = "%s/%s/%s" % (newsid.groups()[2], newsid.groups()[1], newsid.groups()[0])
      if self.options.val('find_author'):
        pos1 = link.find("~")
        pos2 = link.find("/", pos1)
        author = link[pos1+1:pos2]
        if self.options.val('find_id'):
          id = link[pos2+1:-5]
      newitem = RSSItem(id, title, link, date, author, descr, categ)
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
    tmplist.reverse()
    tmplist.sort()
    return tmplist
  
  def process(self, items=[]):
    for rssitem in items:
      if self.firstId is None:
        self.firstId = rssitem.id
      if rssitem.id not in self.objects.keys():
        log.debug("%s: nouvel rssitem: %s" % (self.name, rssitem.id))
        self.objects[rssitem.id] = rssitem
        if self.lastId is not None:
          self.objects[self.lastId].next = rssitem
        self.lastId = rssitem.id
        if self.viewer_open:
          self.viewer.insertItem(rssitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def parseDescr(self, text):
    if self.options.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, self.options.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = toString(get_url_from_base(self.bouchot.options.val('http_server')+self.options.val('backend'), getAttr("src", text[start:end])))
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=self.options.val('encoding'), base_href=self.bouchot.options.val('http_server')+self.options.val('backend'))
    return (msg, tags)
  