# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module dummy_bot
##   Ce module dfinit un plugin-cadre pour des bots posteurs simples
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.persistants import Identity
import re, random

def build_plugins(pycc):
  return { \
    'DummyBot': DummyBot(pycc, "DummyBot", "Un bot tout simple")
  }

# Modes de fonctionnement du Bot pour le postage
DUMMYBOT_FREQ = "dfini par la frquence"      # Mode frquence : le bot postera tous les n posts, selon la frquence dfinie en %
DUMMYBOT_INVOKE = "sur invocation uniquement"  # Mode invocation : le bot ne postera que si on le bigornophone par nom_du_bot<

# Deux classes pour l'interface graphique du bot
class DummyBotEditor(BotOptionsEditor):
  def disp_choice(self, fr, name, option):
    if name == 'post_mode':
      self.widgets[name] = ComboField(fr, option.descr, [DUMMYBOT_FREQ, DUMMYBOT_INVOKE], value=option.get())
      self.widgets[name].pack(pady=2, fill=X)
    else:
      self.widgets[name] = ComboField(fr, option.descr, self.extra_arg.pycc.identities.get().keys(), value=option.get())
      self.widgets[name].pack(pady=2, fill=X)

class DummyBotDialog(BotOptionsDialog):
  def body(self, master):
    self.w_options = DummyBotEditor(master, self.object, extra_arg=self.plugin)
    self.w_options.pack()
  

# Le bot proprement dit
class DummyBot(BotPlugin):
  
  def upd_regexp(self):
    # Mise  jour de la regexp d'invocation du bot, pr-compile pour speeder un peu
    self.regexp = re.compile("".join(["[%s%s]" % (c.upper(), c.lower()) for c in self.options.val('bot_name')]) + "[<]\s*(.*)$")
  
  def init(self):
    self.upd_regexp()
  
  def set_defaults(self):
    # Les options du bot (extension de la dfinition des BotOptions, cf. coinlib/plugin.py
    self.options.set('bot_name', self.name)
    self.options.add('post_mode', Option("Postage", DUMMYBOT_INVOKE, TYPE_CHOICE))
    self.options.add('post_freq', Option("Frquence (0=jamais  100=chaque fois)", 0, TYPE_INT))
    self.options.add('toedit', Option("Editables", { \
          "Gnral": [ 'bot_name', 'bot_id', 'post_mode', 'post_freq' ]
        } ))
  
  def config_frame(self, master):
    # Construction de l'interface d'dition des options du bot
    return DummyBotEditor(master, self.options, self.pycc)
    self.upd_regexp()
  
  def action_menu(self):
    # Quoi faire quand on appelle le bot via le menu plugin : affichage de la bote de dialogue des options
    DummyBotDialog(self.name, self.pycc.root, self.options, self)
    self.upd_regexp()
  
  def start(self, data=None):
    # Lancement du bot : on enregistre le dclenchement du plugin dans le PostProcessor
    # (normalement, il n'y aurait pas eu besoin de redfinir cette mthode qui est dj prsente
    # dans la classe-mre BotPlugin, mais la diffrence est sur les conditions de dclenchement
    # du bot : la mthode originelle ne fonctionne qu'en mode invocation, ie pas sur tous les posts)
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded:
      return
    myfilter, pos = self.find_filter()
    if myfilter is None:
      tribcond = Condition('trib', OP_MATCHES, '(%s)' % cur_trib.getName())
      condset  = ConditionSet([tribcond], COND_ALL)
      action = ActionPlugin(self.name)
      newfilter = Filter("Bot_%s" % self.options.val('bot_name'), [action], condset, 1, 1)
      self.pycc.processor.add_filter(newfilter)
    else:
      triblist = myfilter.conditions.conditions[0].expr[1:-1].split("|")
      if cur_trib.getName() not in triblist:
        triblist.append(cur_trib.getName())
        myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
    if not self.started:
      self.start_process(data)
    self.started = 1
  
  def bot_process(self, message):
    # Le processing proprement dit 
    # On vrifie d'abord si le message matche la regexp d'invocation du bot
    re_parm = self.regexp.match(message)
    if not re_parm:
      # a ne matche pas : on prend en input l'intgralit du post
      input_text = message
    else:
      # a matche : on ne garde que ce qu'il y a aprs "nom_du_bot<"
      input_text = re_parm.groups()[0]
      
    # --- partie  customiser
    result = input_text # Ici insrer le traitement  faire sur "input_text"
    # --- fin partie  customiser
    
    if self.options.val('post_mode') == DUMMYBOT_INVOKE:
      if not re_parm:
        # On est en mode invocation, mais le bot n'a pas t invoqu => on ne renvoie rien
        return None
    else:
      if random.randrange(100) > self.options.val('post_freq'):
        # On est en mode frquence, mais le tirage au sort tombe en-dessous du seuil => on ne renvoie rien
        return None
    # On renvoie ce que le bot doit poster
    return result
  