# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module Pendu
##   Ce module dfinit le plug-in Pendu - un bot pour jouer au pendu.
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')

from constants import *
from coinlib.plugin import *
from coinlib.persistants import Identity, Option, OptionSet, Condition, ConditionSet, Filter, ActionPlugin
from cointklib.dialogs import CoinDialog
from cointklib.basicwidgets import *

import os, os.path, re
import tkMessageBox
from random import randrange, choice
from time import time, sleep

def build_plugins(pycc):
  return { \
    'pendu': Pendu(pycc, "Pendu", "Un bot pour jouer au pendu", "Bots")
  }

ident = Identity(login="Pendu", ua="Pendu")

class PenduStats(OptionSet):
  
  def setName(self, name):
    self.name = "PenduStats"
  
  def setObjects(self):
    self._objects = {
      'stats_file': Option("Fichier de statistiques", value="", type=TYPE_FILE),
      'scores': Option("Tableau des scores", value={}),
      'nb_win': Option("Parties gagnes", value=0),
      'nb_lose': Option("Parties perdues", value=0),
      'avg_eff': Option("efficacit moyenne", value=0),
      'toedit': Option("Editables", { \
          "Gnral": [ 'stats_file', 'nb_win', 'nb_lose', 'avg_eff' ]
        } )
    }
  
class PenduScore(object):
  
  def __init__(self, score, nb_win, nb_lose, eff, login):
    self.score = score
    self.nb_win = nb_win
    self.nb_lose = nb_lose
    self.eff = eff
    self.login = login
  
  def get(self):
    return self.score, self.nb_win, self.nb_lose, self.eff, self.login
  
  def __cmp__(self, other):
    if self.score < other.score:
      return 1
    elif self.score == other.score:
      if self.nb_win < other.nb_win:
        return 1
      elif self.nb_win == other.nb_win:
        if self.nb_lose > other.nb_lose:
          return 1
        elif self.nb_lose == other.nb_lose:
          if float(self.eff) < float(other.eff):
            return 1
    return -1
  

# Etats pour l'enregistrement des points
GAME_LOSE = 0
GAME_WON = 1
GAME_TURN = 2

class PenduServer(object):
  
  def __init__(self, dico_path, trib, plugin_ref, name):
    self.trib = trib
    self.plugin = plugin_ref
    self.dico_path = dico_path
    self.name = name
    self.re_letter = re.compile("^.*[<]\s*([A-Za-z]+)\s*.*$")
    #self.hang_steps = [ "", "O", "|", ".-", "-.", "/", "\\", "RIP", "Gagn!" ]
    self.hang_steps = [ "", "o", "o--", "o-'-", "o-|-", "o-|-'", "o-|-<", "RIP", "Gagn!" ]
    self.server_running = 0
    self.game_running = 0
    self.words = []
    self.lastpost = 0
    self.id = Identity()
    self.id.login = self.trib.bouchot.options.val('default_id')
    # self.load_words(self.dico_path)
  
  def load_words(self, dico_path):
    try:
      f = open(dico_path, "r")
      self.words = [ word[:-1] for word in f.readlines() ]
      f.close()
      self.dico_path = dico_path
      log.debug("PENDU: dictionnaire recharg")
    except:
      tkMessageBox.showwarning("Plugin Pendu", "Impossible de charger le fichier de mots\n%s.\n Vrifiez le chemin." % dico_path)
  
  def start_game(self):
    if self.game_running:
      return # already running
    if len(self.words) == 0:
      log.debug("PENDU: aucun mot dans le dico !")
      return
    log.info("PENDU: nouvelle partie")
    # Initialisation
    self.hidden_word = choice(self.words).lower()  # Choix du mot
    log.debug("PENDU: le mot=%s" % self.hidden_word)
    self.letters = []     # Liste des lettres joues
    self.hang_level = 0   # Niveau de pendaison
    self.game_running = 1 # Boolen partie en cours
  
  def play(self, post):
    if not self.game_running:
      self.start_game()
    # Vrification de la validit du coup jou
    pendu_invoke = self.re_letter.match(post.message)
    if pendu_invoke:
      played_letters = pendu_invoke.groups()[0].lower()
    else:
      played_letters = ""
    if len(played_letters) > len(self.hidden_word):
      played_letters = ""
    # Vrification des lettres nouvellement joues/dj joues
    valid_play = 0
    good_letters = bad_letters = 0
    for letter in played_letters:
      if letter not in self.letters:
        valid_play = 1
        self.letters.append(letter)
        self.letters.sort()
        if letter in self.hidden_word:
          good_letters += 1
        else:
          bad_letters += 1
          self.hang_level += 1  # Mauvaise lettre
    if not valid_play and len(self.letters) > 0:
      log.debug("PENDU: zis is not a valid turn")
      return
    # Calcul du score
    if len(self.letters) > 0:
      #good_letters = len([l for l in self.letters if l in self.hidden_word])
      #bad_letters = len(self.letters) - good_letters
      score = ((100.0*len([l for l in self.letters if l in self.hidden_word]))/len(self.letters))
      s_score = "%2.2f%%" % score
    else:
      score = -1
      s_score = ""
    self.id.ua = self.name # Affectation pour l'user-agent du robot
    if self.hang_level >= 7:
      # Partie perdue
      difftime = time()-self.lastpost
      if difftime >= 0 and difftime < 14:
        sleep(14-difftime)
      self.trib.poster(u"%s [%s] <b>Perdu!</b> C'tait: <tt>%s</tt>" % (post.dispTimeRef(), self.name, self.hidden_word), 1, self.id)
      self.game_running = 0
      self.plugin.record(GAME_LOSE, good_letters, bad_letters, score, post.login)
      self.lastpost = time()
      return
    # Construction du mot avec les lettres trouves
    resword = ""
    for c in self.hidden_word:
      if c in self.letters:
        resword += c
      else:
        resword += '-'
    rec_state = GAME_TURN
    if '-' not in resword:
      # Partie gagne
      rec_state = GAME_WON
      #self.plugin.record(GAME_WON, good_letters, bad_letters, score, post.login)
      self.hang_level = -1
      self.game_running = 0
      self.hang_steps[self.hang_level] = u"%s a gagn!" % post.login 
    # On poste le rsultat
    difftime = time()-self.lastpost
    if difftime >= 0 and difftime < 14:
      sleep(14-difftime)
    self.trib.poster(u"%s [%s] <b>%s</b> <tt> %s </tt> <s>%s</s>  <i>%s</i>" % (post.dispTimeRef(), self.name, self.hang_steps[self.hang_level], resword, "".join([l for l in self.letters if l not in self.hidden_word]), s_score), 1, self.id)
    self.plugin.record(rec_state, good_letters, bad_letters, score, post.login)
    self.lastpost = time()


class PenduOptions(CoinDialog):
  
  def __init__(self, plug, pendu, parent, title, buttons):
    self.plug = plug
    self.pendu = pendu
    CoinDialog.__init__(self, parent, title, buttons)
  
  def body(self, master):
    if self.pendu.server_running:
      Label(master, text="Bouchot %s: Pendu Server actif !" % self.pendu.trib.getName(), background='#80FF80').pack(pady=2, fill=X)
    else:
      Label(master, text="Bouchot %s: aucun Pendu Server dmarr." % self.pendu.trib.getName(), background='#FF8080').pack(pady=2, fill=X)
    self.wdico = FileField(master, "Dictionnaire", value=self.pendu.dico_path)
    self.wdico.pack(pady=2, fill=X)
    self.wname = TextField(master, "Nom pour invocation", value=self.pendu.name)
    self.wname.pack(pady=2, fill=X)
    self.wid = ComboField(master, "Identit pour poster", self.plug.pycc.identities.get().keys(), self.pendu.id.login)
    self.wid.pack(pady=2, fill=X, anchor=SW)
  
  def bind_button(self, button):
    if button == 'Statistiques':
      return self.closeStats
    elif button == 'Lancer':
      return self.closeStart
    else:
      return self.closeStop
  
  def closeStart(self):
    #self.pendu.dico_path = self.wdico.get()
    self.pendu.load_words(self.wdico.get())
    id = self.plug.pycc.identities.get()[self.wid.get()]
    self.pendu.id = Identity()
    self.pendu.id.cookies = id.cookies
    self.pendu.id.login = id.login
    if not self.pendu.server_running:
      self.pendu.name = self.wname.get()
    self.plug.res = 1
    self.cancel()
  
  def closeStop(self):
    self.plug.options.save()
    self.plug.res = 0
    self.cancel()
  
  def closeStats(self):
    self.plug.options.save()
    self.plug.gen_html_stats()
    # print "### PENDU: stats =", "file:///"+self.plug.options.val('stats_file')
    self.plug.pycc.openBrowser(('file:///'+self.plug.options.val('stats_file')).replace(" ", "%20"))
    self.cancel()
  

class Pendu(Plugin):
  
  def init(self):
    self.servers = {}
  
  def load_config(self):
    self.options = PenduStats(path=self.config_path)
    if not self.options.val('stats_file'):
      self.options.set('stats_file', os.path.join(self.pycc.config_path, "pendu_stats.html"))
  
  #def config_frame(self, master):
  #  return TrolloEditor(master, self.options, self.pycc)
  
  def record(self, rec_state, good_letters, bad_letters, eff, plogin=""):
    if plogin:
      login = plogin
    else:
      login = "__anonymes__"
    scores = self.options.val('scores')
    if login not in scores.keys():
      scores[login] = (0, 0, 0, "0.0")
    score, nb_won, nb_lose, avg_eff = scores[login]
    score += (good_letters - bad_letters)
    if rec_state == GAME_WON:
      score += 10
      nb_won += 1
      self.options.set('nb_win', self.options.val('nb_win') + 1)
    elif rec_state == GAME_LOSE:
      score -= 5
      nb_lose += 1
      self.options.set('nb_lose', self.options.val('nb_lose') + 1)
    if rec_state != GAME_TURN:
      avg_eff = "%2.2f" % (((nb_won+nb_lose-1) * float(avg_eff) + eff) / (nb_won+nb_lose))
      total = self.options.val('nb_win') + self.options.val('nb_lose')
      tot_avg_eff = float(self.options.val('avg_eff'))
      self.options.set('avg_eff', "%2.2f" % (((total-1)*tot_avg_eff+eff)/total))
    scores[login] = (score, nb_won, nb_lose, avg_eff)
    #self.options.save()
  
  def gen_html_stats(self):
    stats_file = self.options.val('stats_file')
    html_head = u"""<?xml version="1.0" encoding="iso-8859-15"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr" lang="fr">
  <head>
    <title>pyCoinCoin - Statistiques du plugin Pendu</title>
  </head>
  <body>
    <style type="text/css"><!--
    body {
      font-family: sans-serif;
      color: black;
      background-color: #b7c1ff; 
    }
    table {
      border-style:none;
      width:640px;
      margin-left:auto;
      margin-right:auto;
    }
    th {
      border-style:solid; 
      border-width:1px; 
      border-color:green;
      background-color:#32cd32;
    }
    td { 
      border-style:solid; 
      border-width:1px; 
      border-color:black; 
      background-color:BurlyWood;
      text-align:center;
      color:black;
    }
    -->
    </style>
    <h1>Statistiques du plugin Pendu</h1>
    <table>
      <tr><th>Rang</th><th>Joueur</th><th>Score</th><th>Parties gagnes</th><th>Parties perdues</th><th>Efficacit moyenne</th></tr>
"""
    html_body = ""
    scores = [ PenduScore(pts, nb_win, nb_lose, eff, login) for login, (pts, nb_win, nb_lose, eff) in self.options.val('scores').items() if login != "__anonymes__" ]
    scores.sort()
    cpt = 1
    tot_pts = 0
    for score in scores:
      pts, nb_win, nb_lose, eff, login = score.get()
      html_body += u"      <tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n" % (cpt, login, pts, nb_win, nb_lose, eff)
      cpt += 1
      tot_pts += pts
    try:
      pts, nb_win, nb_lose, eff = self.options.val('scores')['__anonymes__']
      html_body += u"      <tr><td></td><td>Les anonymes</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n" % (pts, nb_win, nb_lose, eff)
      tot_pts += pts
    except:
      pass # si aucun anonyme n'a de score enregistr encore
    nb_win = self.options.val('nb_win')
    nb_lose = self.options.val('nb_lose')
    html_foot = u"""    </table>
    <ul>
      <li>Total parties gagnes&nbsp;: %s</li>
      <li>Total parties perdues&nbsp;: %s</li>
      <li>Score moyen par partie&nbsp;: %2.2f</li>
      <li>Efficacit moyenne des joueurs&nbsp;: %s</li>
    </ul>
  </body>
</html>""" % (nb_win, nb_lose, (tot_pts+0.0)/(nb_win+nb_lose+0.0), self.options.val('avg_eff'))
    f = open(stats_file, "w")
    f.write((html_head+html_body+html_foot).encode('iso-8859-15'))
    f.close()
  
  def action_menu(self):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded: return
    
    if self.servers.has_key(cur_trib.getName()):
      pendu = self.servers[cur_trib.getName()]
    else:
      pendu = PenduServer(os.path.join('plugins', 'pendu.words'), cur_trib, self, "pendu")
      self.servers[cur_trib.getName()] = pendu
    
    # boite de dialogue dico + bouton dmarrer/stopper serveur + bouton recharger dico
    #res = dialog toto...
    self.res = -1
    PenduOptions(self, pendu, self.pycc.root, "Serveur de jeu Pendu", ["Lancer", "Arrter", "Statistiques", BUT_CANCEL])
    
    myfilter = None
    pos = 0
    for filter in self.pycc.processor.get():
      if filter.name == "Pendu_server":
        myfilter = filter
        break
      pos += 1
    if self.res == 1:    # start server
      if myfilter is None:
        tribcond = Condition('trib', OP_MATCHES, '(%s)' % cur_trib.getName())
        penducond = Condition('message', OP_MATCHES, pendu.name+'[<]')
        condset  = ConditionSet([tribcond, penducond], COND_ALL)
        action = ActionPlugin('pendu')
        newfilter = Filter("Pendu_server", [action], condset, 1, 1)
        self.pycc.processor.add_filter(newfilter)
      else:
        triblist = myfilter.conditions.conditions[0].expr[1:-1].split("|")
        if cur_trib.getName() not in triblist:
          triblist.append(cur_trib.getName())
          myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
      pendu.server_running = 1
      pendu.start_game()
    elif self.res == 0:   # stop server
      if myfilter is not None:
        triblist = [trib for trib in myfilter.conditions.conditions[0].expr[1:-1].split("|") if trib != cur_trib.getName()]
        if len(triblist) > 0:
          myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
        else:
          self.pycc.processor.remove_filter(pos)
      del self.servers[cur_trib.getName()]
      pendu.server_running = 0
  
  def process(self, bouchot, context, post=None, *args, **kwargs):
    try:
      self.servers[bouchot.name].play(post)
    except KeyError:
      self.error = "Aucun serveur pendu dmarr sur %s" % bouchot.name
      log.info("PENDU: %s" % self.error)
