# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in Totoz Manager
##   Plugin pour le bookmark et la gestion avance des Totoz
##   auteur: chrisix sur une base de Putifuto
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')

import xml.dom.minidom
import urllib
import md5, os, os.path
import Pmw, tkMessageBox

from coinlib.plugin import *
from cointklib.dialogs import *
from cointklib.widgets import OptionsEditor

def build_plugins(pycc):
  return { \
    'TotozManager': TotozManager(pycc, "Advanced Totoz Manager", "Gestionnaire de smileys (totoz).")
  }

def getText(nodelist):
  rc = ""
  for node in nodelist:
    if node.nodeType == node.TEXT_NODE:
      rc = rc + node.data
  return rc

class TotozDialog(CoinDialog):

  def __init__(self, pycc, parent, plugin, title=None, text='', **kwargs):
    self.pycc = pycc
    self.text = text
    self.plugin = plugin
    self.kwargs = kwargs
    CoinDialog.__init__(self, parent, title, buttons=[BUT_CLOSE])
  
  def body(self, parent):
    self.tabs = Pmw.NoteBook(parent, raisecommand=self.tabselect, tabpos='n', borderwidth=1)
    self.tabs.pack(fill=BOTH, expand=1, padx=0, pady=0)
    
    master = self.tabs.add("Totoz Smileys")
    
    lframe = Frame(master)
    lframe.pack(side=LEFT, fill=BOTH, expand=1)
    rframe = Frame(master)
    rframe.pack(side=LEFT, fill=BOTH, expand=1)
    
    self.w_listb = SortedListBox(lframe, [t[0] for t in self.plugin.options.val('book_totoz')], **self.kwargs)
    self.w_lists = ListBox(lframe, [], **self.kwargs)
    self.lists = []
    
    self.w_ab = Button(rframe, text="Afficher Bookmarks", command=self.toggle_list)
    self.w_ab.pack(pady=5, padx=5, fill=X, expand=1)
    self.w_search = Entry(rframe)
    self.w_search.pack(padx=5, fill=X, expand=1)
    # self.unbind("<Return>")
    self.w_search.bind("<KeyRelease>", self.keypressed)
    self.w_r = Button(rframe, text="Rechercher", command=self.search_totoz)
    self.w_r.pack(anchor=E, padx=5)
    
    lw = Pmw.LabeledWidget(rframe, labelpos='n', label_text='Totoz slectionn :')
    lw.component('hull').configure(relief='ridge', borderwidth=2)
    self.w_rsb = Button(lw.interior(), text="Chercher synonymes", command=self.search_synonyms)
    self.w_rsb.pack(padx=4, pady=0, fill=X, expand=1)
    ifr = Frame(lw.interior())
    ifr.pack(padx=4, pady=0, fill=X, expand=1)
    self.w_atb = Button(ifr, text="Bookmarquer", command=self.supp_or_add)
    self.w_atb.pack(side=LEFT, padx=0, pady=0, fill=X, expand=1)
    self.w_rt = Button(ifr, text="Recharger", command=self.reload)
    self.w_rt.pack(side=LEFT, padx=0, pady=0, fill=X, expand=1)
    self.w_cpb = Button(lw.interior(), text="Copier dans le Palmi", command=self.add_to_palmi)
    self.w_cpb.pack(padx=4, pady=0, fill=X, expand=1)
    lw.pack(padx=5, pady=5, fill=X, expand=1)
    
    self.labl = Label(rframe)
    self.labl.pack(padx=6, pady=6, fill=BOTH, expand=1)
    self.labl.bind("<1>", self.browse_cur_totoz)
    
    self.status = Label(rframe)
    self.status.pack()
    
    master = self.tabs.add("Configuration")
    self.w_options = OptionsEditor(master, self.plugin.options)
    self.w_options.pack(fill=BOTH, expand=1)
    b = Button(master, text="Utiliser ce serveur de totoz\nsur toutes les tribunes", command=self.use_server_on_all)
    b.pack(padx=5, pady=5)
    b = Button(master, text="Sauvegarder la configuration", command=lambda s=self:s.w_options.apply())
    b.pack(padx=5, pady=5)
    b = Button(master, text="Serveur de totoz on ze web", command=self.browse_totoz)
    b.pack(padx=5, pady=5, anchor=S)
    
    self.tabselect("Totoz Smileys")
    self.w_lists.bind_select(self.onSelect)
    self.w_listb.bind_select(self.onSelect)
    self.books_on = 1
    self.disp_bookmarks()
    # self.w_listb.select('totoz')
  
  def tabselect(self, tab):
    if tab == 'Totoz Smileys':
      pass
    elif tab == 'Configuration':
      pass
    self.tabs.selectpage(tab)

  def keypressed(self, event):
    if str(event.keysym) != "Return":
      return
    self.search_totoz()
  
  def toggle_list(self):
    if self.books_on:
      self.disp_search()
    else:
      self.disp_bookmarks()
  
  def supp_or_add(self):
    if self.books_on:
      self.delete()
    else:
      self.add_to_book()
  
  def browse_totoz(self):
    self.pycc.openBrowser("http://"+self.plugin.options.val('totoz_server'))
  
  def disp_bookmarks(self):
    self.books_on = 1
    self.w_lists.pack_forget()
    self.w_listb.pack(padx=6, pady=6, fill=BOTH, expand=1)
    self.w_ab.config(text="Afficher Recherche")
    self.w_atb.config(text="Supprimer")
  
  def disp_search(self):
    self.books_on = 0
    self.w_listb.pack_forget()
    self.w_lists.pack(padx=6, pady=6, fill=BOTH, expand=1)
    self.w_ab.config(text="Afficher Bookmarks")
    self.w_atb.config(text="Bookmarquer")
  
  def get_selected(self):
    if self.books_on:
      totoz = self.w_listb.selected
      ids = [id for name, id in self.plugin.options.val('book_totoz') if name == totoz]
    else:
      totoz = self.w_lists.selected
      ids = [id for name, id in self.lists if name == totoz]
    if len(ids):
      id = ids[0]
    else:
      id = None
    return totoz, id

  def onSelect(self):
    totoz, id = self.get_selected()
    if not totoz:
      return
    self.status.configure(text="Chargement de %s..." % totoz)
    self.update_idletasks()
    if self.pycc.getSmiley(totoz):
      self.labl.configure(image=self.pycc.images[totoz])
      self.status.configure(text="")
    else:
      self.status.configure(text="Erreur sur %s !" % totoz)
  
  def get_infos(self, totoz, id):
    server = self.pycc.options.val('totoz_server')
    url = self.plugin.options.val('totoz_infos') % id
    try:
      f = open_url(server, url, self.pycc.options)
      res = f.read()
      f.close()
    except:
      log.error("Erreur rseau ou http en interrogeant le serveur de totoz")
      self.status.config(text="Erreur HTTP/serveur de totoz")
      return
    pos = res.find("md5sum")
    if pos < 1:
      return
    start = res.find("a href=", pos)
    if start <= pos:
      return
    end = res.find('"', start+10)
    urlmd5 = res[start+19:end]
    return urlmd5
  
  def browse_cur_totoz(self, event=None):
    totoz, id = self.get_selected()
    if not id:
      return
    server = self.pycc.options.val('totoz_server')
    url = self.plugin.options.val('totoz_infos') % id
    self.pycc.openBrowser("http://"+server+url)
  
  def reload(self, event=None):
    totoz, id = self.get_selected()
    if not totoz:
      return
    del self.pycc.images[totoz]
    imgurl = "http://%s/%s.gif" % (self.pycc.options.val('totoz_server'), urllib.quote(totoz))
    cacheimg = md5.new(imgurl).hexdigest()
    imgfile = os.path.join(self.pycc.config_path, "cache", cacheimg)
    try:
      os.unlink(imgfile)
    except:
      pass
    self.onSelect()
  
  def delete(self):
    if not self.w_listb.selected:
      return
    self.w_listb.remove(self.w_listb.selected)
    self.plugin.options.set('book_totoz', self.w_listb.getAll())
    self.plugin.options.save()
    self.status.config(text="%s retir du bookmark" % totoz)
  
  def use_server_on_all(self):
    self.w_options.apply()
    server = self.plugin.options.val('totoz_server')
    trib_list = [ b.trib for b in self.pycc.bouchots.values() if b.trib is not None ]
    for trib in trib_list:
      trib.options.set('smileys_server', server)
      trib.options.save()
    tkMessageBox.showinfo("Totoz Manager Config", "'%s'\nest dsormais utilis comme serveur de Totoz\npour toutes les tribunes." % server, parent=self)
  
  def search_synonyms(self):
    totoz, id = self.get_selected()
    if not totoz:
      return
    md5 = self.get_infos(totoz, id)
    if not md5:
      self.status.config(text="md5 du totoz introuvable !")
      self.update_idletasks()
      return
    server = self.plugin.options.val('totoz_server')
    confurl = self.plugin.options.val('totoz_synonyms')
    url = confurl % md5
    self.w_rsb.config(state=DISABLED)
    self.status.config(text="Interrogation du serveur...")
    self.update_idletasks()
    try:
      f = open_url(server, url, self.pycc.options)
      res = f.read()
      f.close()
    except:
      log.error("Erreur rseau ou http en interrogeant le serveur de totoz")
      self.status.config(text="Erreur HTTP/serveur de totoz")
      self.w_rsb.config(state=NORMAL)
      self.update_idletasks()
      return
    try:
      dom = xml.dom.minidom.parseString(res)
    except:
      log.error("Erreur de parsing XML sur la rponse du serveur de totoz")
      self.status.config(text="Rponse du serveur illisible")
      self.w_rsb.config(state=NORMAL)
      self.update_idletasks()
      return
    reslist = []
    for node in dom.getElementsByTagName("alias"):
      for tag in node.getElementsByTagName("totoz"):
        id = tag.getAttribute("id")
        fntag = tag.getElementsByTagName("filename")[0]
        totoz = getText(fntag.childNodes)
        reslist.append((-9999, totoz[:-4], id))
    dom.unlink()
    self.w_rsb.config(state=NORMAL)
    self.fill_slist(reslist)
  
  def fill_slist(self, reslist):
    reslist.sort() #reverse=1)
    self.w_lists.empty()
    self.lists = []
    for rank, name, id in reslist:
      if type(name) == type(u""):
        totoz = name.encode('utf-8', 'ignore')
      else:
        totoz = name
      self.w_lists.add(totoz)
      self.lists.append((totoz, id))
    self.status.config(text="%s totoz trouv(s)" % len(reslist))
    self.disp_search()
  
  def search_totoz(self):
    totoz = self.w_search.get()
    if not totoz:
      return
    self.w_r.config(text="Recherche en cours...", state=DISABLED)
    self.status.config(text="Interrogation du serveur...")
    self.update_idletasks()
    if totoz[-1] == "\n":
      totoz = totoz[:-1]
    if type(totoz) == type(u""):
      totoz = totoz.encode('utf-8', 'ignore')
    server = self.pycc.options.val('totoz_server')
    url = self.plugin.options.val('totoz_search') % urllib.quote(totoz)
    try:
      f = open_url(server, url, self.pycc.options)
      res = f.read()
      f.close()
    except:
      log.error("Erreur rseau ou http en interrogeant le serveur de totoz")
      self.status.config(text="Erreur HTTP/serveur de totoz")
      self.w_r.config(text="Rechercher", state=NORMAL)
      self.update_idletasks()
      return
    try:
      dom = xml.dom.minidom.parseString(res)
    except:
      log.error("Erreur de parsing XML sur la rponse du serveur de totoz")
      self.status.config(text="Rponse du serveur illisible")
      self.w_r.config(text="Rechercher", state=NORMAL)
      self.update_idletasks()
      return
    reslist = []
    for node in dom.getElementsByTagName("resultsByName"):
      for tag in node.getElementsByTagName("totoz"):
        id = tag.getAttribute("id")
        fntag = tag.getElementsByTagName("filename")[0]
        totoz = getText(fntag.childNodes)
        reslist.append((-9999, totoz[:-4], id))
    for node in dom.getElementsByTagName("resultsByTag"):
      for tag in node.getElementsByTagName("totoz"):
        fntag = tag.getElementsByTagName("filename")[0]
        totoz = getText(fntag.childNodes)
        if totoz[:-4] not in [t[1] for t in reslist]:
          id = tag.getAttribute("id")
          poptag = tag.getElementsByTagName("popularity")[0]
          score = int(getText(poptag.childNodes))
          reslist.append((score*-1, totoz[:-4], id))
    dom.unlink()
    self.w_r.config(text="Rechercher", state=NORMAL)
    self.fill_slist(reslist)
  
  def add_to_book(self):
    if not self.w_lists.selected:
      return
    totoz, id = self.get_selected()
    lst = self.plugin.options.val('book_totoz')
    if totoz not in [name for name, key in lst]:
      lst.append((totoz, id))
      self.plugin.options.set('book_totoz', lst)
      self.plugin.options.save()
      self.w_listb.add(totoz)
      self.w_listb.update()
      self.status.config(text="%s bookmarqu" % totoz)
    else:
      self.status.config(text="%s est dj en bookmark" % totoz)
  
  def add_to_palmi(self):
    if self.books_on:
      totoz = self.w_listb.selected
    else:
      totoz = self.w_lists.selected
    if not totoz:
      return
    self.plugin.pycc.palmi.palmi.inserter('[:%s] ' % totoz)
  

class TotozManagerOptions(OptionSet):
  def setName(self, name):
    self.name = "TotozManager"

  def setObjects(self):
    self._objects = {  
      'totoz_server': Option("Serveur", "totoz.eu"),
      'totoz_search': Option("URL de recherche", "/totoz/search?q=%s&xml=true"),
      'totoz_infos': Option("URL d'infos dtailes", "/totoz/show/%s"),
      'totoz_synonyms': Option("URL des synonymes", "/totoz/md5xml/%s"),
      'book_totoz': Option("Totoz bookmarkes", [], TYPE_LIST),
      'toedit': Option("Editables", {"General": ['totoz_search', 'totoz_synonyms', 'totoz_infos', 'totoz_server']})
    }

class TotozManager(Plugin):
  def load_config(self):
    # lit le bookmark
    log.info("load config smiley selector")
    self.options = TotozManagerOptions(path=self.config_path)
    self.options.save()

  def config_frame(self, master):
    return OptionsEditor(master, self.options, self.pycc)

  def action_menu(self):
    TotozDialog(self.pycc, self.pycc.root, self, "Totoz Smileys Manager", 
                     "Advanced Totoz Smileys Manager", width=24, height=30)
  
  def process(self, bouchot, context, post=None, keyword=None, ctxt_tags=[], *args, **kwargs):
    # if not bouchot.trib.options.val('disp_smileys'): return
    if post is None: return
    smiley_list = [ tag for tag in ctxt_tags if tag[:7] == "smiley_" ]
    if not smiley_list:
      smiley_list = [ tag for tag in post.tags.keys() if tag[:7] == "smiley_" ]
    lst = self.options.val('book_totoz')
    for tag in smiley_list:
      if tag[7:] not in lst:
        lst.append(tag[7:])
    self.options.set('book_totoz', lst)
    self.options.save()
