# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in Trollomtre
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.persistants import Option, OptionSet
from cointklib.widgets import OptionsEditor
from cointklib.basicwidgets import *

def build_plugins(pycc):
  return { \
    'trollo': Trollo(pycc, "Trollomtre", "Calcule un coefficient indicateur du potentiel trollistique du post")
  }


class TrolloOptions(OptionSet):
  def setName(self, name):
    self.name = "Trollo"
  
  def setObjects(self):
    self._objects = {
      'trollo_words': Option("Coefficient trollistique", value={ \
       # Les trolls classiques
        u'troll':8, u'su><or':5, u'suxor':8, u'mdk':2, u'merdrake':10, u'pwet':10, u'ro><or':2, u'non':1,
        u'tusor':4, u'boul':5,
       # Les trolls vi/emacs
        u'vi':3, u'emacs':3, u'mieux':10, u'troll':5,
       # GNU/Linux
        u'gnu/linux': 10, u'intaigriste':4, u'hurd':5, u'gnu/hurd':10,
        u'kde':3, u'gnome':5, u'moz':4, u'ie ':3, u'ze ':3, u'beos':5,
        u'php': 7, u'daubian':15, u'debian':2, u'gentoo': 6,
       # les grossiers
        u'connard':5, u'ta gueule':7, u'connerie':5,'pd':9,' tapaite':15, u' con ':2, u'merde':4,
       # Les gros trolls godwin et politique
        u'guerre':5, u'juifs':10, u'nazi':12, u'palestin':10, u'godwin': 5, u'chrtiens':10, u'jsus':9, u'dieu':8,
        u'raciste':7, u'allemand':3, u'communiste':6, u'drouate':2,
       # les trolls m$
        u'm$':9, u'microsoft':8, u'virus':6, u'daube':9, u'format c:':16, u' xp ':8,
       # les sexuelles
        u'sesque ':5, u' bite':10, u'suce':15, 
       # une question, c'est louche
        u'??':1, u'rtfm':9, u'lol':1, u'!!':3, u'mais ':1, u'marche pas':5, u'plus ':1, u'moins ':4
      }, type=TYPE_LIST ),
      'toedit':    Option("Editables", { \
          "Gnral": [ 'trollo_words' ]
        } )
    }

class TrolloEditor(OptionsEditor):
  
  def body(self):
    OptionsEditor.body(self)
    fr = Frame(self.tab)
    self.trollword = TextField(fr, "Mot-troll:")
    self.trollword.pack(side=LEFT)
    self.coeff = IntField(fr, "coeff")
    self.coeff.pack(side=LEFT)
    Button(fr, text="Ajouter", command=self.new).pack(side=LEFT)
    fr.pack(anchor=W)
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    objlist = [ "%s : %s" % (word, coeff) for word, coeff in option.get().items() ]
    self.widgets[name] = SortedListBox(fr, objlist, width=40, height=10)
    self.widgets[name].pack(fill=X)
    # self.widgets[name].bind_new(self.new)
    # self.widgets[name].bind_edit(self.edit)
  
  def new(self):
    word = self.trollword.get()
    coeff = self.coeff.get()
    if not word or not coeff:
      return
    self.widgets['trollo_words'].add("%s : %s" % (word, coeff))
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'trollo_words':
        myhash = {}
        for rule in widget.getAll:
          word, coeff = rule.split(" : ")
          myhash[word] = int(coeff)
        self.object.set(opt, myhash)
      else:
        self.object.set(opt, widget.get())
    self.object.save()

class Trollo(AttribPlugin):
  
  def init(self):
    self.attrib = 'trollo'
    self.attpos = 0
  
  def load_config(self):
    self.options = TrolloOptions(path=self.config_path)
  
  def config_frame(self, master):
    return TrolloEditor(master, self.options, self.pycc)
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, font=('Courier', 8, 'bold'), foreground='#ff0000')
  
  def compute_attrib(self, post):
    i = trollo = 0
    text = post.message.lower()
    for word, coeff in self.options.val('trollo_words').items():
      if text.find(word) > -1:
        trollo += coeff
      i += 1
    score = trollo + int(10*trollo/i)
    if score > 0:
      return "%2d" % score
    else:
      return "  "
  